% Approximate a 2D Gaussian by different landmark methods

[X,Y] = meshgrid(-1:.05:1, -1:.05:1);
[ny,nx]=size(X) ;
Z = exp(-(X.^2 + Y.^2)*6);
figure(1) ; clf ;
mesh(X,Y,Z) ; title('Function to approximate') ; colormap(jet) ;
print -deps aprgausfn.eps

N=15 ; % number of sampling points
figure(2) ; clf ;
if 0,
  lnd=[ 2*rand(N,2)-1 ] ;
  % random samples + border points
  lnd=[ lnd ; -1 -1 ; -1 1 ; 1 -1 ; 1 1 ]  ; 
  lnd=[ lnd ; 0 1 ; 1 0 ; 0 -1 ; -1 0 ] ;
  %lnd=[ lnd ; -1 0.5 ; -1 -0.5 ; 1 0.5 ; 1 -0.5 ] ;
  %lnd=[ lnd ; 0.5 -1 ; -0.5 -1 ; 0.5 1 ; -0.5 1 ] ;
  save aprgauslnd lnd
else
  load aprgauslnd
end ;

plot(lnd(:,1),lnd(:,2),'o') ; title('Sampling points') ;
axis equal
print -deps aprgaussmpl.eps

lndz=exp(-(lnd(:,1).^2 + lnd(:,2).^2)*6);
ct=lndthscoef(lnd,lndz) ;

zt=Z ;
for i=1:nx,
  for j=1:ny,
    zt(j,i)=lndthseval([Y(j,i) X(j,i)],lnd,ct) ;
  end ;
end ;

figure(3) ;
mesh(X,Y,zt) ; title('Thin plate spline approximation') ;
colormap(jet)


difscale=0.2 ; 
figure(4) ;
%subplot(232) ; colormap(jet) ;
%imagesc(Z,[0 1]) ; title('original') ;
%subplot(233) ; 
igray=gray ;
igray=igray(length(igray):-1:1,:) ;
colormap(igray)
imagesc(zt,[0 1]) ; title('thin plate') ; 
%subplot(234) ; 
imagesc(abs(zt-Z),[0 difscale]) ; % title('difference') ; 
colorbar ; 
axis image ; colormap(igray)
print -deps aprgausdiftp.eps

% Derivative approximation

lndzd=[ lndz -12*lndz.*lnd(:,2) -12*lndz.*lnd(:,1) ] ;
cz=lndthscoefder(lnd,lndzd) ;
zd=Z ;
for i=1:nx,
  for j=1:ny,
    zd(j,i)=lndthsdereval([Y(j,i) X(j,i)],lnd,cz) ;
  end ;
end ;

figure(5) ;
mesh(X,Y,zd) ; title('Derivative approximation') ;
colormap(jet)


figure(6) ;
%subplot(235) ; colormap(jet) ;
imagesc(zd,[0 1]) ; title('der') ;
%subplot(236) ; 
imagesc(abs(zd-Z),[0 difscale]) ; % title('difder') ; 
colorbar ; axis image ; colormap(igray) ;
print -deps aprgausdifder.eps

% Interpolating with r^4 log(r) but without derivatives
c4=lndthscoefder(lnd,lndz) ;
nl=length(c4)-6 ; 
c4=[ c4(1:nl) ; zeros(2*nl,1) ; c4(nl+1:nl+6) ] ;
z4=Z ;
for i=1:nx,
  for j=1:ny,
    z4(j,i)=lndthsdereval([Y(j,i) X(j,i)],lnd,c4) ;
  end ;
end ;

figure(7) ;
mesh(X,Y,z4) ; title('r4 log r') ;
colormap(jet)
%print -deps aprgausr4.eps
figure(8)
imagesc((z4-Z),[0 difscale]) ; % title('difder') ; 
colorbar ; axis image ; colormap(igray) 
print -deps aprgausdifr4.eps


sne=sum(sum(Z.^2)) ;
snrzt=-10*log(sum(sum((zt-Z).^2))/sne)
snrzd=-10*log(sum(sum((zd-Z).^2))/sne)
snrz4=-10*log(sum(sum((z4-Z).^2))/sne)