function c=lndradcoef(x,z,alpha) ;
% Calculate coefficients for radial power function interpolation
% Calculates coefficients (usable e.g. by lndradeval) to interpolate
% function having values z at points x
%
% Usage: c=lndradcoef(x,z,alpha) 
%
% $Id: lndradcoef.m,v 1.2 1999/10/08 12:24:53 jkybic Exp $
% Jan Kybic, 1999

n=length(z) ;
A=zeros(n+3) ;
alphah=alpha/2 ;

for i=1:n,
  for j=1:n,
    r2=(x(i,1)-x(j,1))^2+(x(i,2)-x(j,2))^2 ;
    if r2==0,
      A(i,j)=0 ;
    else
      A(i,j)=r2^alphah ; 
    end ;
  end ;
  A(i,n+1:n+3)=[ x(i,1:2) 1 ] ; 
end ;

% Adding the linear part
A(n+1,1:n)=x(:,1)' ;
A(n+2,1:n)=x(:,2)' ;
A(n+3,1:n)=ones(1,n) ;

% Solve using SVD techniques
[u,s,v]=svd(A) ; 
s=diag(s) ; q=(s(1)*1e-20+1e-50) ;
for i=1:n+3,
  if s(i)<q, s(i)=0 ; 
  else s(i)=1/s(i) ; end ;
end ;

c=v*diag(s)*u'*[z(:) ; 0 ; 0 ; 0 ] ;