/* lndradtbl.c - (interface to Matlab through mexlndradtbl)

   Given a radial power function knots and coefficients,
   evaluate this function in a table (0,ny)x(0,nx)

   void lndradtbl(double *xp,double *cp,double *yp,int n, int ny, 
               int nx, double alpha) ;

   where x is the n x 2 matrix of landmark points, cp are the coefficients, 
   yp the output table (ny times nx).    Uses fi(r)= r^alpha   basis functions

   Jan Kybic, October 1999

   Uses C coordinates, that is starting at zero, and x-first.

   $Id: lndradtbl.c,v 1.2 1999/10/08 12:24:55 jkybic Exp $
*/

#include <math.h>

void lndradtbl(double *xp,double *cp,double *yp,int n, int ny, 
               int nx, double alpha) 
{ 
  int i,j,k ; double sum,t1,t2,r ;
  
  alpha*=0.5 ; 
  for(j=0;j<ny;j++) { /* iterate over rows */
    for(i=0;i<nx;i++) { /* iterate over columns */
      sum=cp[n+2]+cp[n]*i+cp[n+1]*j ; /* the linear part */
      /* mexPrintf("linear part %g\n",sum) ; */
      for(k=0;k<n;k++) { /* loop over all basis functions */
	t1=i-xp[k] ; t2=j-xp[k+n] ; r=t1*t1+t2*t2 ;
	sum+=cp[k]*pow(r,alpha) ;
        /* mexPrintf("adding %g %g %g\n",r,pow(r,alpha),cp[k]*pow(r,alpha)) ; */
      }
      yp[i+nx*j]=sum ;
     }
   }
}
