function [c,cnd]=lndthscoef(x,z) ;
% Calculate coefficients for thin-plate spline interpolation
% Calculates coefficients (usable e.g. by lndthseval) to interpolate
% function having values z at points x
%
% Usage: c=lndthscoef(x,z) 
%
% $Id: lndthscoef.m,v 1.3 2000/01/27 18:11:53 jkybic Exp $
% Jan Kybic, 1999

n=length(z) ;
A=zeros(n+3) ;

for i=1:n,
  for j=1:n,
    r2=(x(i,1)-x(j,1))^2+(x(i,2)-x(j,2))^2 ;
    if r2==0,
      A(i,j)=0 ;
    else
      A(i,j)=r2*log(r2) ; % actually twice as much as r^2 ln(r) 
    end ;
  end ;
  A(i,n+1:n+3)=[ x(i,1:2) 1 ] ; 
end ;

A(n+1,1:n)=x(:,1)' ;
A(n+2,1:n)=x(:,2)' ;
A(n+3,1:n)=ones(1,n) ;

% Solve using SVD techniques
[u,s,v]=svd(A) ;
s=diag(s) ; 
cnd=s(1)/s(n+1) ;
q=(s(1)*1e-20+1e-50) ; % the q-value is sensitive !!!
for i=1:n+3,
  if s(i)<q, s(i)=0 ; 
  else s(i)=1/s(i) ; end ;
end ;

c=v*diag(s)*u'*[z(:) ; 0 ; 0 ; 0 ] ;