/* mexlndradtbl.c MEX file source

   Given a radial power function knots and coefficients,
   evaluate this function in a table (0,ny)x(0,nx)

   Usage: y=mexlndradtbl(x,c,ny,nx,alpha)

   Jan Kybic, October 1999

   Uses fi(r)= r^alpha   basis functions

   Uses C coordinates, that is starting at zero, and x-first.

   $Id: mexlndradtbl.c,v 1.2 1999/10/08 12:24:55 jkybic Exp $
*/


#define myCreateMatrix(a,b) mxCreateDoubleMatrix(a,b,mxREAL)
#define myAlloc(a) mxMalloc(a)

#include <math.h>
#include "mex.h"

#define X_IN	 prhs[0]
#define C_IN	 prhs[1]
#define	NY_IN	 prhs[2]
#define	NX_IN	 prhs[3]
#define	ALPHA_IN prhs[4]
#define Y_OUT   plhs[0]

#define NAME "MEXLNDRADTBL"

#define max(a,b) ((a>b) ? (a) : (b))

void lndradtbl(double *xp,double *cp,double *yp,int n, int ny, 
               int nx, double alpha) ;

void mexFunction(int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{ int nx,ny,n ; int i ;
  double *xp,*cp,*yp,alpha ;
 
  if (nrhs != 5) {
    mexErrMsgTxt(NAME " requires five input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }
  
  for (i=0;i<nrhs;i++)
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(
             NAME " needs all input arguments to be real.");
     } 
 
  ny=(int)*(mxGetPr(NY_IN)) ; 
  nx=(int)*(mxGetPr(NX_IN)) ; 
  n=mxGetM(X_IN) ;
  cp=mxGetPr(C_IN) ;  xp=mxGetPr(X_IN) ;  
  alpha=*mxGetPr(ALPHA_IN) ; 

  Y_OUT = myCreateMatrix(ny,nx); yp=mxGetPr(Y_OUT) ;
 /*  %%%%% here the calculation begins %%%%%% */
  lndradtbl(xp,cp,yp,n,nx,ny,alpha) ;
  /* note the swap between nx,ny */
} 

#include "lndradtbl.c"




