% Show derivative interpolation
% $Id: showderivint.m,v 1.1 1999/10/12 14:12:49 jkybic Exp $

% cubic splines minimising second derivatives in 1D

% Let first define points to interpolate 

x=[ 3 ; 5 ; 6 ; 12 ; 17 ; 23 ] ;
y=[ 1 ; 2 ; 3 ; 6 ; 3 ; 2 ] ;
dy=[ 0 ; 0.2 ; -0.2 ; 0 ; -1 ; 1 ] ;
  
n=length(x) ;

% set up the system of equations
A=zeros(n+2) ;

for i=1:n, % generate the system matrix
  for j=1:n,
    A(i,j)=abs(x(i)-x(j))^3 ; 
  end ;
  A(i,n+1:n+2)=[ x(i) 1 ] ; 
end ;

A(n+1,1:n)=x(:)' ;
A(n+2,1:n)=ones(1,n) ;

c=A\[y ; 0 ; 0] ;

% where we evaluate 
tx=1:0.1:25 ;
yx=tx ; % interpolated value

m=length(tx) ;
for i=1:m, % for all points
  yx(i)=c(n+2)+c(n+1)*tx(i)+c(1:n)'*abs(x-tx(i)).^3 ;
end ;

figure(1) ;
plot(tx,yx,'b-',x,y,'ro') ; 
axis([ 0 25 0 7]) ;
print -deps showderivint-fig1.eps


%%%% and now for the derivative interpolation

% set up the system of equations
Ad=zeros(2*n+2) ;

for i=1:n, % generate the system matrix
  for j=1:n,
    Ad(i,j)=abs(x(i)-x(j))^3 ;
    Ad(i,j+n)=3*(x(i)-x(j))*abs(x(i)-x(j)) ;
  end ;
  Ad(i,2*n+1:2*n+2)=[ x(i) 1 ] ; 
end ;

for i=n+1:2*n, % generate the second half of the system matrix
  for j=1:n,
    Ad(i,j)=3*(x(i-n)-x(j))*abs(x(i-n)-x(j)) ;
    Ad(i,j+n)=6*abs(x(i-n)-x(j)) ;
  end ;
  Ad(i,2*n+1:2*n+2)=[ 1 0 ] ; 
end ;

Ad(2*n+1,1:2*n)=[ -x' ones(1,n) ] ;
Ad(2*n+2,1:n)=ones(1,n) ;

ccd=Ad\[y ; dy ; 0 ; 0] ;
yxd=yx ;

for i=1:m, % for all points
  yxd(i)=ccd(2*n+2)+ccd(2*n+1)*tx(i)+ccd(1:n)'*abs(x-tx(i)).^3+...
      +3*ccd(n+1:2*n)'*(abs(tx(i)-x).*(tx(i)-x)) ;
end ;

figure(2) ;
h=0.8 ;
% plot(tx,yxd,'b-.',x,y,'ro',[x(1)-h x(1)+h],[y(1)-h*dy(1) y(1)+h*dy(1)],'r-') ; 
plot(tx,yxd,'b',x,y,'ro') ;
axis([ 0 25 0 7]) ;
line([x-h x+h]',[y-h*dy y+h*dy]','Color','black','LineWidth',2) ;
print -deps showderivint-fig2.eps
