% Show motivation for variational interpolation
% $Id: showvarmot.m,v 1.2 2001/01/26 16:00:56 jkybic Exp $

% cubic splines minimising second derivatives in 1D

% Let first define points to interpolate 

x=[ 3 ; 5 ; 6 ; 12 ; 17 ; 23 ] ;
y=[ 1 ; 2 ; 3 ; 6 ; 3 ; 2 ] ;
dy=[ 0 ; 0.2 ; -0.2 ; 0 ; -1 ; 1 ] ;
  
n=length(x) ;

% set up the system of equations
A=zeros(n+2) ;

for i=1:n, % generate the system matrix
  for j=1:n,
    A(i,j)=abs(x(i)-x(j))^3 ; 
  end ;
  A(i,n+1:n+2)=[ x(i) 1 ] ; 
end ;

A(n+1,1:n)=x(:)' ;
A(n+2,1:n)=ones(1,n) ;

c=A\[y ; 0 ; 0] ;

% where we evaluate 
tx=1:0.1:25 ;
yx=tx ; % interpolated value

m=length(tx) ;
for i=1:m, % for all points
  yx(i)=c(n+2)+c(n+1)*tx(i)+c(1:n)'*abs(x-tx(i)).^3 ;
end ;

zx=tx ; czx=tx ;
zx(1)=1 ; sigma=0.1 ;
czx(1)=1 ;
for i=2:m,
  q=find(x==tx(i)) ;
  if length(q)==0,
    zx(i)=zx(i-1)+randn(1)*sigma ;
    czx(i)=czx(i-1)*0.8 ;
  else
    zx(i)=y(q) ; czx(i)=1 ; 
  end ;
end ;

bx=tx ; cbx=tx ;
bx(m)=3 ; sigma=0.1 ; 
cbx(m)=1 ;
for i=m-1:-1:1,
  q=find(x==tx(i)) ;
  if length(q)==0,
    bx(i)=bx(i+1)+randn(1)*sigma ;
    cbx(i)=cbx(i+1)*0.8 ;
  else
    bx(i)=y(q) ; cbx(i)=1 ;
  end ;
end ;

zbx=(zx.*czx+bx.*cbx)./(czx+cbx) ;


figure(1) ;
plot(x,y,'ko',tx,zbx,'k-',tx,yx,'k-') ; 
axis([ 0 25 0 7]) ;
hh=findobj(gca,'Type','line')
set(hh(3),'LineWidth',4) ; % points
set(hh(1),'LineWidth',2) ; % smooth
legend('Given points','Rugged interpolation','Smooth interpolation') ;
print -deps showvarmot-fig1.eps


