function [z,E,gr,Hd]=lpeval(x,c,T,d,p,deg)
% Evaluate a spline of degree deg, calculate the l_p norm of the difference
% with x, its gradient and second derivatives d^2 E/d c_k^2
% x ... vector to approximate at positions 0,1,2,...,N-1, 
% c ... Bspline coefficients c_0, c_1, ..., 
% T ... spline spacing, d ... spline displacement, 
% p... norm order, deg ...spline degree,
% z ... spline values at 1..N=length(x), z_i= sum_k c_k B_deg(i/T-k+d)
% E ... sum_i ( z_i - y_i )^p
% gr_k = d E/ d c_k
% Hd_k = d gr_k/ d c_k      (diagonal of the Hess matrix)
%
% Jan Kybic, 1999
% $Id: lpeval.m,v 1.3 2000/01/07 18:32:33 jkybic Exp $

x=x(:) ; c=c(:) ; % make column vectors
N=length(x) ;
M=length(c) ;

t=((1:N)-1)/T+d ;
z=mexsplninterp(c,t,1,deg)' ; % column vector
E=sum(abs(z-x).^p) ;
if p==1,
  w1=sign(z-x) ; w2=zeros(N,1) ;
elseif p==2,
  w1=2*(z-x) ;
  w2=2*sign(z-x) ;
else
  w1=p*sign(z-x).*(abs(z-x)).^(p-1) ;
  w2=p*(p-1)*sign(z-x).*(abs(z-x)).^(p-2) ;
end ;
gr=zeros(M,1) ;
Hd=zeros(M,1) ;
for i=1:M,
%  bs=mexsplninterpone(i-1,M,t,1,deg) ; % gives a row vector
  bs=splninterpone(i-1,M,t,1,deg) ; % gives a row vector
  gr(i)=bs*w1 ;
  Hd(i)=bs.^2*w2 ;
end ;