function c=lpmgrid(x,c0,T,d,p,deg,level,maxlev,f)
% Optimize the lpeval criterion with respect to c using multigrid method
% x ... vector to approximate at positions 0,1,2,...,N-1, 
% c0,c ... Bspline coefficients c_0, c_1, ..., 
%          initial estimate (c0) and the final value (c)
% T ... spline spacing, d ... spline displacement, 
% p... norm order, deg ...spline degree,
%
% level ... the level of the multiresolution we are at. 1-original, 2-coarser...
%
% Usage: c=lpmgrid(x,c0,T,d,p,deg,[level,maxlevel]) 
%
% Jan Kybic, January 2000
% $Id: lpmgrid.m,v 1.5 2000/01/31 17:48:50 jkybic Exp $

% if we are called without sufficient number of arguments, remedy it 
if nargin==6, % 
  % as many levels as to stop at about length 4 for c0
  % we suppose x is longer
  maxlev=max(1,ceil(log(length(c0))/log(2)-2)) ;
  f=zeros(length(x),1) ; % right hand side is zero at the beginning
  c=lpmgrid(x,c0,T,d,p,deg,1,maxlev,f) ;
  return ;
end ;

omega=1 ;
nsmpre=2 ;  % How many presmoothing iterations at normal level 
nsmpost=2 ; % How many postsmoothing iterations at normal level 
nsmc=200 ;% How many smoothing iterations at the coarsest level 

if level==maxlev, % the last level
  c=c0 ;
  for i=1:nsmc,
    [c,omega]=smooth(f,x,c,T,d,p,deg,omega,f) ;
  end ;
  return ;
end ;

% ordinary iterations begins with presmoothing
c=c0 ;
for i=1:nsmpre,
  [c,omega]=smooth(f,x,c,T,d,p,deg,omega,f) ;
end ;

% end with postsmoothing
for i=1:nsmpost,
  [c,omega]=smooth(f,x,c,T,d,p,deg,omega,f) ;
end ;


function [c,newomega]=smooth(f,x,c0,T,d,p,deg,omega)
% Implements non-linear Gaussian smoothing
% with adaptive step-size
[z,Eold,gr]=lpeval(x,c0,T,d,p,deg) ; 
[z,E]=lpeval(x,c0-omega*(gr-f),T,d,p,deg) ; 
uns=0 ;
while E>Eold, % if unsuccessfull, decrease step size
  omega=omega/2 ;
  [z,E]=lpeval(x,c0-omega*(gr-f),T,d,p,deg) ;
  uns=1 ;
end ;
newomega=omega ;
if uns==0,
  while E<Eold, % if successfull, increase step size
    newomega=omega ; Eold=E ;
    omega=newomega*2 ;
    [z,E]=lpeval(x,c0-omega*(gr-f),T,d,p,deg) ; 
  end ;
  E=Eold ;
end ;
E
c=c0-newomega*gr ;

function y=restrict(x)
y=dyaddown(conv(x,[1 2 1]/4),1) ;
