function [x,J]=graddesc(critfun,x0,param)
% Gradient descent minimization method
% critfun - criterion function, e.g. 'crit', will be called
% with two arguments - a vector x and additional parameters param,
% or just with x, if param is empty.
% critfun should return two output arguments: 
% the criterion and
% its gradient with respect to x.
% x0 is the starting guess
%
% Usage: x=graddesc(critfun,x0,[param])
%
% $Id: graddesc.m,v 1.6 1999/12/05 18:55:49 jkybic Exp $
% Jan Kybic, 1999

x=x0(:) ;
prec=1e-4 ;     % relative precision
absprec=1e-20 ; % absolute precision
alphamin=1e-10 ; 
global conjgraddofine conjgraditer
conjgraddofine=1 ;
conjgraditer=1 ;

if nargin<3, param=[] ; end ;

i=0 ;
[J,gr]=feval(critfun,x,param) ;
disp(['Iteration ' num2str(i) ' J=' num2str(J) ]) ;
if conjgraddofine,
  evalfinecrit(x,conjgraditer) ;
  conjgraditer=conjgraditer+1 ;
end ;
i=i+1 ;
J0=2*J+1 ;
alpha=1 ;

while (J0-J)>prec*J0 & (J0-J)>absprec,
  xt=x-alpha*gr ;
  [Jt,grt]=feval(critfun,xt,param) ;
  disp(['Iteration ' num2str(i) ' Jt=' num2str(Jt) ' alpha=' num2str(alpha)]) ;
  i=i+1 ; j=0 ;
  if conjgraddofine,
    evalfinecrit(xt,conjgraditer) ;
    conjgraditer=conjgraditer+1 ;
  end ;
  while Jt>J & alpha>alphamin,
    alpha=0.1*alpha ;
    xt=x-alpha*gr ;
    Jt=feval(critfun,xt,param) ;
    disp(['Inner iteration ' num2str(i) ' Jt=' num2str(Jt) ' alpha=' num2str(alpha)]) ;
    i=i+1 ; j=1 ;
    if conjgraddofine,
      evalfinecrit(xt,conjgraditer) ;
      conjgraditer=conjgraditer+1 ;
    end ;
  end ;
  if j>0, % if needed, evaluate gradient
    [Jt,grt]=feval(critfun,xt,param) ;
    disp(['Gradient evaluation ' num2str(i) ' Jt=' num2str(Jt) ' alpha=' num2str(alpha)]) ;
    i=i+1 ;
    if conjgraddofine,
      evalfinecrit(xt,conjgraditer) ;
      conjgraditer=conjgraditer+1 ;
    end ;
  else
    alpha=alpha*2 ;
  end ;
  J0=J ; J=Jt ; gr=grt ; x=xt ;
end ;



