function out=L2pyramid_down(in,n, spacein, spaceout)
% L2PYRAMID_DOWN     REDUCE an image by factor 2
%   OUT=L2PYRAMID_DOWN(IN,N) downsamples the image in the DUAL basis
%   by a factor of 2 using spline filtering of degree n.
%   If no input or output space are given cardinal space is used.
%   
%   SPACEIN:  input  image  can be in 'cardinal', 'dual' or 'bspline' space.
%   SPACEOUT: output image  can be in 'cardinal', 'dual' or 'bspline' space.
%
%   Reference: IEEE Vol15, No.4, p 371, fig 3.
%   Author: Stefan Horbelt, 23.4.1998
%
%   $Id: L2pyramid_down.m,v 1.6 1999/11/18 21:04:31 jkybic Exp $
%   changes by JK, 1999

xsize=size(in,1);
ysize=size(in,2);
zsize=size(in,3) ;

%if(mod(xsize,2)~=0 | mod(ysize,2)~=0 | xsize*ysize<=4)
%  error('ERROR: image size must be multiple of 2'); 
%end;

if(nargin<3)  spacein='card'; end;
if(nargin<4)  spaceout='card';end;

% GO TO DUAL SPACE:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%cc=cputime ;
img=spspace(in,n,spacein,'dual');
%disp(['to dual ' num2str(cputime-cc) ' s' ]) ;

% REDUCE (dual basis) prefilter: 1/2*u2^n
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
u=bio_coeff(n);

%cc=cputime ;

img=firfiltm(img,u/2) ;
if zsize==1,
  img=dyaddown(img,'m',1);
else % 3D case
  for i=1:ceil(zsize/2),
    img(1:ceil(xsize/2),1:ceil(xsize/2),i)=...
	dyaddown(img(:,:,2*i-1),'m',1) ;
  end ;
  img=img(1:ceil(xsize/2),1:ceil(xsize/2),1:ceil(zsize/2)) ;
end ;

%disp(['filtering ' num2str(cputime-cc) ' s' ]) ;

% GO FROM DUAL TO SPACEOUT: (cardinal by default)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%cc=cputime ;
out=spspace(img,n,'dual',spaceout);
%disp(['from dual ' num2str(cputime-cc) ' s' ]) ;



 
