function out=L2pyramid_up(in,n, spacein,spaceout)
% L2PYRAMID_UP     EXPAND an image by factor 2
%   OUT=L2PYRAMID_UP(IN,N) upsamples the image in the B-SPLINE basis
%   by a factor of 2 using spline filtering of degree n.
%   If no input or output space are given cardinal space is used.
%   
%   SPACEIN:  input  image  can be in 'cardinal', 'dual' or 'bspln' space.
%   SPACEOUT: output image  can be in 'cardinal', 'dual' or 'bspln' space.
%
%   Reference: IEEE Vol15, No.4, p 371, fig 3.
%   Author: Stefan Horbelt, 23.4.1998
%
%   $Id: L2pyramid_up.m,v 1.7 1999/11/18 21:04:32 jkybic Exp $
%   changes by JK, 1999

xsize=size(in,1);
ysize=size(in,2);
zsize=size(in,3) ;

img=zeros(xsize,ysize,zsize);
up=zeros(xsize*2,ysize*2-(ysize==1),2*zsize-(zsize==1));

if(nargin<3) spacein='card'; end;
if(nargin<4) spaceout='card'; end;

% GO TO BSPLINE SPACE:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%cc=cputime ;
img=spspace(in,n,spacein,'bspl');
%disp(['to bspln ' num2str(cputime-cc) ' s' ]) ;

% EXPAND (B-spline basis)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

u=bio_coeff(n);

%cc=cputime ;
if zsize>1, % 3D case
  for i=1:zsize,
    q=dyadup(img(:,:,i),'m',1) ;
    up(:,:,2*i-1)=q(2:xsize*2+1,2:ysize*2+1) ;
  end ;
elseif  ysize>1, % 2D case
  up=dyadup(img,'m',1);
  up=up(2:xsize*2+1,2:ysize*2+1);
else % 1D case
  up=dyadup(img,'r',1) ;
  up=up(2:xsize*2+1);
end ;
up=firfiltm(up,u) ;
%disp(['filtering ' num2str(cputime-cc) ' s' ]) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%ok


% GO FROM BSPLINE SPACE TO OUTSPACE (card.):
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%cc=cputime ;
out=spspace(up,n,'bspl',spaceout);
%disp(['from bspln ' num2str(cputime-cc) ' s' ]) ;


