function u=bio_coeff(n);
% BIO_COEFF  returns bionomial filter coefficients 
%   BIO_COEFF(N) calculates the bionomial filter coefficients at
%   the order N and return an vector U with them.
%   Attention: The filter is not symmetrical.
%   Application: u=bio_coeff(n); 
%                filfirs(SIGNAL,u);
%   Reference: IEEE Vol15, No.4, p 370, eq (4.6)
%   Author: Stefan Horbelt, 23.4.1998

% Allocation:

  if( mod(n,2)~=1)
    error('ERROR: bio_coeff(n): order n must be odd');
  end;

  u=zeros((n+1)/2,1);
  i=1;

% Loop
  for k=0*ceil(-(n+1)/2):floor((n+1)/2)
    u(i)=nchoosek(n+1,k+(n+1)/2)/power(2,n);
    i=i+1;
  end;
