function [c,cm]=bpgetcoefcbwav(thetav,nuh,du,mts,T,gamma)
% Calculates the reconstruction coefficients using turned and shifted 
% cubic B-splines wavelets for the tomographical reconstruction problem
%
% Usage: c=bpgetcoefcbwav(thetav,nu,du,mts,T,gamma)
%
% T is the integration length, thetav angles, 
% uv offsets (there are 2*nuh-1 of them, du apart, the middle one is at
% zero) mts measurements
% measurements mts should be of the size zeros(length(thetav),length(uv)) 
%
% cm are the theoretical measurement values using c, it should be equal to
% mts, if the system matrix is conditioned enough.
%
% Jan Kybic, March 2000

nt=length(thetav) ;
nu=2*nuh-1 ; ;

% setup the system matrix
N=nu*nt ; % number of unknowns
if 1,
  A=zeros(N,N) ;
  for i1=1:nu, % sampling point
    for i2=1:nt,
      i=i1+nu*(i2-1) ;
      uv=(i1-nuh)*du ;
      limT=T^2-uv^2 ;
      if limT>0, limT=sqrt(limT) ; else limT=0 ; end ;
      for j1=1:nu, % coefficient index
	for j2=1:nt,
	  j=j1+nu*(j2-1) ;
	  A(i,j)=intcbwav(thetav(j2),thetav(i2),(j1-nuh)*du,uv,limT,du) ;
	end ;
      end ;
    end ;
  end ;
else
  A=mexbpmatcbspln(thetav,nuh,du,T) ;
end ;



%B=A ;
%
%nxy=ceil(2*T) ;
%for j1=1:nu,
%  for j2=1:nt,
%    j=j1+nu*(j2-1) ;
%    c=cz ; c(j)=1 ;
%    img=mexbpevalcbspln(thetav,nuh,du,c,[nxy nxy]) ;
%    Rp=radonpix(img.*mask,thetav,
    
disp('System matrix known, now solving') ;
 keyboard
r=reshape(mts',nu*nt,1)  ;

%[u,s,v]=svd(A) ;
%s=diag(s) ;
%q=(s(1)*1e-6+1e-50) ; % the q-value is sensitive !!!
%for i=1:length(s),
%  if s(i)<q, s(i)=0 ; 
%  else s(i)=1/s(i) ; end ;
%end ;

%c=v*diag(s)*u'*r ;

  c=A\r ;
cond(A)
  cm=A*c ;
res=sum(abs(cm-r))  % the residual

function val=intcbspln(th1,th2,u1,u2,T,du)
tmp=intxthr(th1,th2,u1-2*du,u2,T) ;
tmp=tmp-4*intxthr(th1,th2,u1-du,u2,T) ;
tmp=tmp+6*intxthr(th1,th2,u1,u2,T) ;
tmp=tmp-4*intxthr(th1,th2,u1+du,u2,T) ;
tmp=tmp+intxthr(th1,th2,u1+2*du,u2,T) ;
val=tmp/du^3 ;

function val=intcbwav(th1,th2,u1,u2,T,du)
tmp=intxthr(th1,th2,u1-2.5*du,u2,T) ;
tmp=tmp-5*intxthr(th1,th2,u1-1.5*du,u2,T) ;
tmp=tmp+10*intxthr(th1,th2,u1-0.5*du,u2,T) ;
tmp=tmp-10*intxthr(th1,th2,u1+0.5*du,u2,T) ;
tmp=tmp+5*intxthr(th1,th2,u1+1.5*du,u2,T) ;
tmp=tmp-intxthr(th1,th2,u1+2.5*du,u2,T) ;
val=tmp/du^3 ;


function val=intxthr(th1,th2,u1,u2,T) ;
% Integrate |-sin(th1)*x+cos(th1)*y-u1|^3/12
% along (cos(th2)*t-sin(th2)*u2,sin(th2)*t+cos(th2)*u2) for t=-T..T
thdif=th1-th2 ;
alpha=abs(sin(thdif)) ; beta=cos(thdif)*u2-u1 ;
if alpha==0,
  val=T*abs(beta)^3/6 ;
elseif -alpha*T+beta>0,
  val=(T*beta*(alpha^2*T^2+beta^2))/6 ;
elseif alpha*T+beta<0,  
  val=-(T*beta*(alpha^2*T^2+beta^2))/6 ;
else
  val=((alpha*T+beta)^4+(-alpha*T+beta)^4)/4/abs(alpha)/12 ;
end ;

function [val1,valx,valy]=intlpol(th2,u2,T) ;
% Integrate (1,x,y) along
% (cos(th2)*t-sin(th2)*u2,sin(th2)*t+cos(th2)*u2) for t=-T..T
val1=2*T ;
valx=-2*T*sin(th2)*u2 ;
valy=+2*T*cos(th2)*u2 ;
 