function c=bpgetcoefsob(thetav,uv,mts,T,gamma)
% Calculates the reconstruction coefficients using turned and shifted
% Sobolev interpolation for the tomographical reconstruction problem
%
% Usage: c=bpgetcoefsob(thetav,uv,mts,T,gamma)
%
% T is the integration length, thetav angles, uv offsets, mts measurements
% measurements mts should be of the size zeros(length(thetav),length(uv)) 
%
% Jan Kybic, March 2000

s=gamma^(-1/4)/sqrt(2) ;
alpha=1/4/sqrt(2)/s^3 ;

nt=length(thetav) ;
nu=length(uv) ;

% setup the system matrix
N=nu*nt ; % number of unknowns
A=zeros(N,N) ;
for i1=1:nu, % sampling point
  for i2=1:nt,
    i=i1+nu*(i2-1) ;
    for j1=1:nu, % coefficient index
      for j2=1:nt,
	j=j1+nu*(j2-1) ;
	thdif=thetav(j2)-thetav(i2) ;
	lls=cos(thdif)*uv(i1)-uv(j1) ;
	if thdif==0,
	  sr=s*abs(lls) ;
	  A(i,j)=2*T*alpha*(exp(-sr)*(sin(sr+pi/4))) ;
	else
	  ll=-sin(thdif)*T+lls; hl=+sin(thdif)*T+lls ;
	  sr=s*abs(ll) ;
	  lli=alpha*(1-exp(-sr)*cos(sr))/sqrt(2)/s ;
	  sr=s*abs(hl) ;
	  hli=alpha*(1-exp(-sr)*cos(sr))/sqrt(2)/s ;
	  if ll*hl<0,
	    tmp=lli+hli ;
	  elseif abs(hl)>abs(ll),
	    tmp=hli-lli ;
	  else
	    tmp=lli-hli ;
	  end ;
	  A(i,j)=tmp/abs(sin(thdif)) ;
	end ;
      end ;
    end ;
  end ;
end ;

r=[ reshape(mts',nu*nt,1) ] ;
c=A\r ;

