function c=bpgetcoefxfrs(thetav,uv,mts,T)
% Calculates the reconstruction coefficients using turned and shifted |x|^3 
% interpolation for the tomographical reconstruction problem
%
% Usage: c=bpgetcoefxfrs(thetav,uv,mts,T)
%
% T is the integration length, thetav angles, uv offsets, mts measurements
% measurements mts should be of the size zeros(length(thetav),length(uv)) 
%
% Jan Kybic, March 2000

nt=length(thetav) ;
nu=length(uv) ;

% setup the system matrix
N=nu*nt+1 ; % number of unknowns
A=zeros(N,N) ;
for i1=1:nu, % sampling point
  for i2=1:nt,
    i=i1+nu*(i2-1) ;
    limT=sqrt(T^2-uv(i1)^2) ;
    for j1=1:nu, % coefficient index
      for j2=1:nt,
	j=j1+nu*(j2-1) ;
%	A(i,j)=intxthr(thetav(j2),thetav(i2),uv(j1),uv(i1),T) ;
%	A(i,j)=intxthr(thetav(j2),thetav(i2),uv(j1),uv(i1),limT) ;
	A(i,j)=intxfrs(thetav(j2),thetav(i2),uv(j1),uv(i1),limT) ;
      end ;
    end ;
%    [val1,valx,valy]=intlpol(thetav(i2),uv(i1),T) ;
    val1=intlpol(thetav(i2),uv(i1),limT) ;
    A(i,nu*nt+1)=val1 ;
  end ;
end ;

A(nu*nt+1,:)=A(:,nu*nt+1)' ;
r=[ reshape(mts',nu*nt,1) ; 0 ] ;
c=A\r ;

function val=intxfrs(th1,th2,u1,u2,T) ;
% Integrate |-sin(th1)*x+cos(th1)*y-u1|/2
% along (cos(th2)*t+sin(th2)*u2,sin(th2)*t-cos(th2)*u2) for t=-T..T
thdif=th1-th2 ;
alpha=abs(sin(thdif)) ; beta=-cos(thdif)*u2-u1 ;
if alpha==0,
  val=T*abs(beta) ;
elseif -alpha*T+beta>0,
  val=beta*T ;
elseif alpha*T+beta<0,  
  val=-beta*T ;
else
  val=(alpha*T^2+2*abs(beta)*T)/2 ;
end ;


function [val1,valx,valy]=intlpol(th2,u2,T) ;
% Integrate (1,x,y) along
% (cos(th2)*t+sin(th2)*u2,sin(th2)*t-cos(th2)*u2) for t=-T..T
val1=2*T ;
valx=T^2*cos(th2)+2*T*sin(th2)*u2 ;
valy=T^2*sin(th2)-2*T*cos(th2)*u2 ;
 