function [c,cm]=bpgetcoefxthr(thetav,uv,mts,T,gamma)
% Calculates the reconstruction coefficients using turned and shifted |x|^3 
% interpolation for the tomographical reconstruction problem
%
% Usage: c=bpgetcoefxthr(thetav,uv,mts,T[,gamma])
%
% T is the integration length, thetav angles, uv offsets, mts measurements
% measurements mts should be of the size zeros(length(thetav),length(uv)) 
%
% cm are the theoretical measurement values using c, it should be equal to
% mts, if the system matrix is conditioned enough. gamma is the
% regularization factor, to be added to the matrix
%
% Jan Kybic, March 2000

nt=length(thetav) ;
nu=length(uv) ;

if nargin<5, gamma=0 ; end ;
N=nu*nt+3 ; % number of unknowns

% setup the system matrix
if 0, % Matlab code
  N=nu*nt+3 ; % number of unknowns
  A=zeros(N,N) ;
  for i1=1:nu, % sampling point
    for i2=1:nt,
      i=i1+nu*(i2-1) ;
      limT=T^2-uv(i1)^2 ;
      if limT>0, limT=sqrt(limT) ; else limT=0 ; end ;
      for j1=1:nu, % coefficient index
	for j2=1:nt,
	  j=j1+nu*(j2-1) ;
	  % A(i,j)=intxthr(thetav(j2),thetav(i2),uv(j1),uv(i1),T) ;
	  A(i,j)=intxthr(thetav(j2),thetav(i2),uv(j1),uv(i1),limT) ;
	end ;
      end ;
      %    [val1,valx,valy]=intlpol(thetav(i2),uv(i1),T) ;
      [val1,valx,valy]=intlpol(thetav(i2),uv(i1),limT) ;
      A(i,nu*nt+1)=val1 ;
      A(i,nu*nt+2)=valx ;
      A(i,nu*nt+3)=valy ;
    end ;
  end ;
A(nu*nt+1:nu*nt+3,:)=A(:,nu*nt+1:nu*nt+3)' ;

else % C code
  A=mexbpmatxthr(thetav,uv,T) ;
end ;
%keyboard
NN=nu*nt ;
A(1:NN,1:NN)=A(1:NN,1:NN)+eye(NN)*gamma ; % regularization
disp('System matrix known, now solving') ;
r=[ reshape(mts',nu*nt,1) ; 0 ; 0 ; 0 ] ;
save bpgetcoefxthrA A r

  
%[u,s,v]=svd(A) ;
%s=diag(s) ;
%q=(s(1)*1e-30+1e-50) ; % the q-value is sensitive !!!
%for i=1:length(s),
%  if s(i)<q, s(i)=0 ; 
%  else s(i)=1/s(i) ; end ;
%end ;

%c=v*diag(s)*u'*r ;

max(max(abs(A)))
  c=A\r ;
% cond(A)
  cm=A*c ;
res=max(abs(cm-r))  % the residual

function val=intxthr(th1,th2,u1,u2,T) ;
% Integrate |-sin(th1)*x+cos(th1)*y-u1|^3
% along (cos(th2)*t-sin(th2)*u2,sin(th2)*t+cos(th2)*u2) for t=-T..T
thdif=th1-th2 ;
alpha=abs(sin(thdif)) ; beta=cos(thdif)*u2-u1 ;
if alpha==0,
  val=T*abs(beta)^3/6 ;
elseif -alpha*T+beta>0,
  val=(T*beta*(alpha^2*T^2+beta^2))/6 ;
elseif alpha*T+beta<0,  
  val=-(T*beta*(alpha^2*T^2+beta^2))/6 ;
else
  val=((alpha*T+beta)^4+(-alpha*T+beta)^4)/4/abs(alpha)/12 ;
end ;

function [val1,valx,valy]=intlpol(th2,u2,T) ;
% Integrate (1,x,y) along
% (cos(th2)*t-sin(th2)*u2,sin(th2)*t+cos(th2)*u2) for t=-T..T
val1=2*T ;
valx=-2*T*sin(th2)*u2 ;
valy=+2*T*cos(th2)*u2 ;
 