function [c,cm]=bpgetcoefzspln(thetav,nuh,du,mts,T)
% Calculates the reconstruction coefficients using turned and shifted 
% cubic B-splines for the tomographical reconstruction problem
%
% Usage: c=bpgetcoefzspln(thetav,nu,du,mts,T)
%
% T is the integration length, thetav angles, 
% uv offsets (there are 2*nuh-1 of them, du apart, the middle one is at
% zero) mts measurements
% measurements mts should be of the size zeros(length(thetav),length(uv)) 
%
% cm are the theoretical measurement values using c, it should be equal to
% mts, if the system matrix is conditioned enough.
%
% Jan Kybic, March 2000

nt=length(thetav) ;
nu=2*nuh-1 ; ;

% setup the system matrix
N=nu*nt ; % number of unknowns
if 0,
  A=zeros(N,N) ;
  for i1=1:nu, % sampling point
    for i2=1:nt,
      i=i1+nu*(i2-1) ;
      uv=(i1-nuh)*du ;
      limT=T^2-uv^2 ;
      if limT>0, limT=sqrt(limT) ; else limT=0 ; end ;
      for j1=1:nu, % coefficient index
	for j2=1:nt,
	  j=j1+nu*(j2-1) ;
	  A(i,j)=intzspln(thetav(j2),thetav(i2),(j1-nuh)*du,uv,limT,du) ;
	end ;
      end ;
    end ;
  end ;
else
  A=mexbpmatzspln(thetav,nuh,du,T) ;
end ;

%B=mexbpmatzspln(thetav,nuh,du,T) ;
disp('System matrix known, now solving') ;
%keyboard
r=reshape(mts',nu*nt,1)  ;

%[u,s,v]=svd(A) ;
%s=diag(s) ;
%q=(s(1)*1e-30+1e-50) ; % the q-value is sensitive !!!
%for i=1:length(s),
%  if s(i)<q, s(i)=0 ; 
%  else s(i)=1/s(i) ; end ;
%end ;

%c=v*diag(s)*u'*r ;

  c=A\r ;
cond(A)   % condition number
  cm=A*c ;
res=sum(abs(cm-r))  % the residual


function val=intzspln(th1,th2,u1,u2,T,du) ;
% Integrate a zero order B-spline centered at level v, at angle th1
% along (cos(th2)*t-sin(th2)*u2,sin(th2)*t+cos(th2)*u2) for t=-T..T
thdif=abs(th1-th2) ;
val=intsec(thdif,u1-du/2,u2,T)-intsec(thdif,u1+du/2,u2,T) ;

function val=intsec(th,v,u,T)
% integrate part of the circle above level v, 
% along (cos(th)*t-sin(th)*u,sin(th)*t+cos(th)*u) for t=-T..T

if th==0,
  if u<v, val=0 ; else val=2*T ; end ;
else
  val=T+(u*cos(th)-v)/sin(th) ;
  if val<0, val=0 ; end ;
  if val>2*T, val=2*T ; end ;
end ;


 