/*
  mexintgrimg

  Evaluates an integral over a line (x1,y1)->(x2,y2) of an image
  All coordinates use Matlab convention (start at 1, y,x) 
  Values outside of the image are assumed to be zero.

  Usage: val=mexintgrimg(v,img) ;

  v=[y1 x1 y2 x2]...the integration path
  img...input image

  $Id: mexintgrimg.c,v 1.1 2000/04/11 16:33:09 jkybic Exp $

  Jan Kybic, April 2000
*/

#include <math.h>
#include "mex.h"

/* define a few macros to keep the other source files generic */
#define myErrMsg mexErrMsgTxt
#define NAME "mexintgrimg"
#define myMalloc(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(double)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myMallocInt(ptr,a,b) \
 if ((ptr=(void *)mxMalloc((a)*(b)*sizeof(int)))==NULL) \
    myErrMsg(NAME ": could not malloc") ;
#define myFree(x) mxFree(x)


double intgrimg(double y1,double x1, double y2, double x2, double *img,
	     int dimy,int dimx) ;


/* Input Arguments */

#define	V_IN	   prhs[0]
#define	IMG_IN     prhs[1]

/* Output Arguments */

#define	VAL_OUT	   plhs[0]

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ double *v, *img, *val ;
  int i,dimy,dimx ;

  if (nrhs != 2) { /* Check for proper number of arguments */
    mexErrMsgTxt(NAME " requires two input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }

  for (i=0;i<nrhs;i++) /* Check for the type */
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(NAME " needs all input arguments to be real.");
     } 
  
  v=mxGetPr(V_IN) ;
  if (mxGetM(V_IN)*mxGetN(V_IN)!=4) 
    mexErrMsgTxt(NAME ": input vector v should have length 4") ;
  img=mxGetPr(IMG_IN) ;
  dimx=mxGetN(IMG_IN) ;
  dimy=mxGetM(IMG_IN) ;

  if ((VAL_OUT=mxCreateDoubleMatrix(1,1,mxREAL))==NULL) 
    mexErrMsgTxt(NAME " Could not create array.") ;
  val=mxGetPr(VAL_OUT) ;

  /* mexPrintf("Calling bpevalthrimg with nthetav=%d nuv=%d nc=%d dimx=%d dimy=%d\n", nthetav,nuv,nc,dimx,dimy) ;
   */
  *val=intgrimg(v[0],v[1],v[2],v[3],img,dimy,dimx) ;
}

double intgrimg(double y1,double x1, double y2, double x2, double *img,
	     int dimy,int dimx) 
{ double xd,yd,ll,xa,ya,val,xstep,ystep ;
  int steps,t,ix,iy ;
 
  xd=x2-x1 ; yd=y2-y1 ;
  ll=sqrt(xd*xd+yd*yd) ;
  steps=3*(int)(ll) ; 
  xstep=xd/steps ; ystep=yd/steps ; 
  val=0.0 ; xa=x1-1 ; ya=y1-1 ; /* Matlab starts at 1, C at 0 */
  for (t=0;t<steps;t++) {
    ix=(int)xa ; iy=(int)ya ;
    if (iy>=0 && iy<dimy && ix>=0 && ix<dimx) val+=img[iy+dimy*ix] ;
    xa+=xstep ; ya+=ystep ;
  }
  val*=ll/steps ;
  return val ;
}

