function R=radonpix(img,thetav,uv,T) ;
% Calculate Radon line integrals for image img, angles thetav (in radians)
% and displacement uv. When the displacement is 0, the integration line
% passes through the center of the image ((1+nx)/2,(1+ny)/2)
%
% Jan Kybic, March 2000
%
% $Id: radonpix.m,v 1.6 2000/04/27 17:45:07 jkybic Exp $

[ny,nx]=size(img) ;
nt=length(thetav) ;
nu=length(uv) ;
R=zeros(nt,nu) ;

if nargin<4, 
  T=1.1*sqrt(ny^2+nx^2) ;
end ;

xc=(1+nx)/2  ; yc=(1+ny)/2 ;

for ni=1:nt,
  for iu=1:nu,
    theta=thetav(ni) ; u=uv(iu) ;
    sint=sin(theta) ; cost=cos(theta) ;
    x1=xc+T*cost-u*sint ; y1=yc+T*sint+u*cost ;
    x2=xc-T*cost-u*sint ; y2=yc-T*sint+u*cost ;
    % R(ni,iu)=intgrimg(img,x1,y1,x2,y2) ;
    R(ni,iu)=mexintgrimg([y1 x1 y2 x2],img) ;
  end ;
end ;
