function R=radonxthr(c,thetav,uv,T,tol) ;
% Calculate Radon line integrals for image described by a linear combination
% of turned and shifted |x|^3 functions, with angles thetav (in radians)
% and displacement uv. When the displacement is 0, the integration line
% passes through the center of the image ((1+nx)/2,(1+ny)/2)
% step is the integration step, should be smaller than zero
% T is the radius of the circle outside which the function is considered to
% be zero
%
% Jan Kybic, April 2000
%
% $Id: radonxthr.m,v 1.2 2000/04/12 18:10:19 jkybic Exp $

nt=length(thetav) ;
nu=length(uv) ;
R=zeros(nt,nu) ;

for ni=1:nt,
  for iu=1:nu,
    theta=thetav(ni) ; u=uv(iu) ;
    sint=sin(theta) ; cost=cos(theta) ;
    lT=sqrt(T^2-u^2) ;
    x1=lT*cost-u*sint ; y1=lT*sint+u*cost ;
    x2=-lT*cost-u*sint ; y2=-lT*sint+u*cost ;
    % R(ni,iu)=intgrimg(img,x1,y1,x2,y2) ;
    R(ni,iu)=mexintgrxthr([y1 x1 y2 x2],thetav,uv,c,tol) ;
  end ;
end ;
