function R=radonxthrexact(c,thetav,uv,T) ;
% Calculate Radon line integrals for image described by a linear combination
% of turned and shifted |x|^3 functions, with angles thetav (in radians)
% and displacement uv. When the displacement is 0, the integration line
% passes through the center of the image ((1+nx)/2,(1+ny)/2)
% step is the integration step, should be smaller than zero
% T is the radius of the circle outside which the function is considered to
% be zero
%
% Jan Kybic, April 2000
%
% $Id: radonxthrexact.m,v 1.2 2000/04/26 18:06:46 jkybic Exp $

nt=length(thetav) ;
nu=length(uv) ;
R=zeros(nt,nu) ;

for ni=1:nt,
  for iu=1:nu,
    theta=thetav(ni) ; u=uv(iu) ;
    sint=sin(theta) ; cost=cos(theta) ;
    lT=sqrt(T^2-u^2) ;
    %x1=lT*cost-u*sint ; y1=lT*sint+u*cost ;
    %x2=-lT*cost-u*sint ; y2=-lT*sint+u*cost ;
    % R(ni,iu)=intgrimg(img,x1,y1,x2,y2) ;
    % R(ni,iu)=mexintgrxthr([y1 x1 y2 x2],thetav,uv,c,tol) ;
    val=0 ;
    for nj=1:nt,
      for ju=1:nu,
	val=val+c(ju+(nj-1)*nu)*...
	       intxthr(thetav(nj),theta,uv(ju),u,lT) ;
      end ;
    end ;
    [val1,valx,valy]=intlpol(theta,u,lT) ;
    val=val+c(nt*nu-2)*val1+c(nt*nu-1)*valx+c(nt*nu)*valy ;
    R(ni,iu)=val ;  
  end ;
end ;

function val=intxthr(th1,th2,u1,u2,T) ;
% Integrate |-sin(th1)*x+cos(th1)*y-u1|^3
% along (cos(th2)*t-sin(th2)*u2,sin(th2)*t+cos(th2)*u2) for t=-T..T
thdif=th1-th2 ;
alpha=abs(sin(thdif)) ; beta=cos(thdif)*u2-u1 ;
if alpha==0,
  val=T*abs(beta)^3/6 ;
elseif -alpha*T+beta>0,
  val=(T*beta*(alpha^2*T^2+beta^2))/6 ;
elseif alpha*T+beta<0,  
  val=-(T*beta*(alpha^2*T^2+beta^2))/6 ;
else
  val=((alpha*T+beta)^4+(-alpha*T+beta)^4)/4/abs(alpha)/12 ;
end ;

function [val1,valx,valy]=intlpol(th2,u2,T) ;
% Integrate (1,x,y) along
% (cos(th2)*t-sin(th2)*u2,sin(th2)*t+cos(th2)*u2) for t=-T..T
val1=2*T ;
valx=-2*T*sin(th2)*u2 ;
valy=+2*T*cos(th2)*u2 ;
 