function tryinyfdi
% We verify that your integration routines are correct

T=2 ;
th1=pi/7 ; th2=-pi/6 ;
u1=1 ; u2=-1.5 ;

val=intfdi(th1,th2,u1,u2,T) 
st=sqrt(T^2-u1^2) ;
valint=quad('tryintfdievalintp',-st,st,1e-3,1,th1,th2,u1,u2,T)

%a=-1 ; b=3.4321; ; p1=1.7 ; p2=1.8 ; c=0 ;
%a=-1 ; b=3.4321; ; p1=1.7 ; p2=1.8 ; c=0 ;
%val=intu1(a,b,p2)-intu1(a,b,p1) 
%valint=quad('tryintfdievalintp',p1,p2,1e-6,1,a,b,u1,u2,T)
%figure(2) 
%t=1.7:0.001:1.74 ; 
%for i=1:length(t),
%  y(i)=intu1(a,b,t(i)) ; 
%  y(i)=(b/(t(i)+a*(b+a*t(i)))) ;
%end ; 
%plot(t,y) 

function val=intfdi(th1,th2,u1,u2,T) ;
% Calculates the integral of the finite domain optimal function
% corresponding to angle th2 and offset u2 along a line with angle
% th1 and offset u1. All inside a circle of diameter T
thdif=th1-th2 ;
st=sqrt(T^2-u1^2) ;
val=intI(thdif,u1,u2,T,st)-intI(thdif,u1,u2,T,-st) ;

function val=intI(thdif, q,t,T,p) ;
sint=sin(thdif) ; cost=cos(thdif) ;
alpha2=cost ; alpha3=-q*sint ; % I change here x and y
alpha0=sint ; alpha1=q*cost ;
if alpha0~=0,
  val=intv1(alpha2/alpha0,-alpha2*alpha1/alpha0+alpha3,...
      p*alpha0+alpha1,t,T)/alpha0 ;
else
  val=intv2(alpha1,p*alpha2+alpha3,t,T)/alpha2 ;
end ;

function val=intv1(a,b,p,t,T)
% Evaluate primitive function to v(x,ax+b) at point x=p
st=sqrt(T^2-t^2) ;
val=intu1(a,a*t+b+st,p-t)/2-intu1(a,a*t+b-st,p-t)/2 ;

function val=intv2(c,p,t,T)
% Evaluate primitive function to v(c,x) at point x=p
st=sqrt(T^2-t^2) ;
val=intu2(c-t,p+st)/2-intu2(c-t,p-st)/2 ;

function val=intu1(a,b,p)
% Evaluate the primitive function of u(x,ax+b) at x=p
%if a<0, a=-a ; p=-p ; end ;
if p~=0,
  val=(-24*p^4*atan(a+b/p)+...
      (-24*b^4*atan((p+a*(b+a*p))/b)-...
      6*a*(3+a^2)*b^4*...
      log(b^2+2*a*b*p+(1+a^2)*p^2)+...
      (1+a^2)*p*(12*(2+a^2)*b^3+18*(a+a^3)*b^2*p+4*(2+5*a^2+3*a^4)*b*p^2+...
      3*a*(3+4*a^2+a^4)*p^3-6*(1+a^2)*(4*b^3+6*a*b^2*p+4*(1+a^2)*b*p^2+...
      a*(3+a^2)*p^3)*log(p^2+(b+a*p)^2)))/(1+a^2)^2)/576/pi ;
else
  val=((24*b^4*atan(b/(a*b)))-6*a*(3+a^2)*b^4*log(b^2))/(1+a^2)^2/576/pi ;
end ;

function val=intu2(c,p)
% Evaluate the primitive function of u(c,x) at x=p
if c~=0,
  val=(10*c^2*p^2+p^4-32*c^3*p*atan(p/c)-...
      2*(-3*c^4+6*c^2*p^2+p^4)*log(c^2+p^2))/192/pi ;
else
  val=(p^4-2*(p^4)*log(p^2))/192/pi ;
end ;  
