function [tbl,dspl]=deftable(c,imsize,degree)
% Creates a 1-3D deformation table tbl of size imsize given by coefs c
%
% Usage: [tbl,dspl]=deftable(c,imsize,degree)
%
% dspl is tbl minus identity
%
% Jan Kybic, 1999
% $Id: deftable.m,v 1.7 1999/11/16 15:19:51 jkybic Exp $

ndim=length(imsize) ; % number of dimensions

if ndim>3 | ndim<1, 
  error('Unsupported dimensionality') ;
end ;

if degree<0 | degree>3,
  error('Unsupported degree') ; 
end ;  

shift=max(0,(degree-1)/2) ;

if ndim==1, 
  N=length(c) ;
  c=c(:) ;
  tbl=zeros(imsize,1) ; 
  step=(N-1-2*shift)/(imsize-1) ;
  t=shift:step:N-1-shift ;
  idt=(0:imsize-1)' ;
  dspl=mexsplninterp(c,t,ndim,degree)' ;
else 
  tbl=zeros(imsize) ;
  N=size(c) ; N=N(2:ndim+1) ;
  step=(N-1-2*shift)./(imsize-1) ;
  if ndim==2,
    if step(1)>0, yv=shift:step(1):N(1)-1-shift ;
    else        yv=shift*ones(1,imsize(1)) ;
    end ;
    if step(2)>0, xv=shift:step(2):N(2)-1-shift ;
    else        xv=shift*ones(1,imsize(2)) ;
    end ;
    
    [x,y]=meshgrid(xv,yv) ;
    t=shiftdim( reshape([y x],imsize(1),imsize(2),2),2) ;
    [x,y]=meshgrid(0:imsize(2)-1,0:imsize(1)-1) ;
    idt=shiftdim( reshape([y x],imsize(1),imsize(2),2),2) ;
    dspl=idt ;
    dspl(1,:,:)=mexsplninterp(squeeze(c(1,:,:)),t,ndim,degree) ;
    dspl(2,:,:)=mexsplninterp(squeeze(c(2,:,:)),t,ndim,degree) ;
  else % ndim==3
    xv=shift:step(1):N(1)-1-shift ;
    yv=shift:step(2):N(2)-1-shift ;
    zv=shift:step(3):N(3)-1-shift ;
    [x,y,z]=meshgrid(yv,xv,zv) ;
    t=zeros(3,length(xv),length(yv),length(zv)) ;
    t(1,:,:,:)=y ; t(2,:,:,:)=x ; t(3,:,:,:)=z ;
    [x,y,z]=meshgrid(0:imsize(2)-1,0:imsize(1)-1,0:imsize(3)-1) ;
    idt=zeros([ ndim imsize ]) ;      dspl=idt ;
    idt(1,:,:,:)=y ; idt(2,:,:,:)=x ; idt(3,:,:,:)=z ; 
    dspl(1,:,:,:)=mexsplninterp(squeeze(c(1,:,:,:)),t,ndim,degree) ;
    dspl(2,:,:,:)=mexsplninterp(squeeze(c(2,:,:,:)),t,ndim,degree) ;
    dspl(3,:,:,:)=mexsplninterp(squeeze(c(3,:,:,:)),t,ndim,degree) ;
  end ;
end ;

tbl=dspl+idt ;

  
