function [tbl,dspl]=deftableone(csize,cpos,imsize,degree)
% Creates a 1-3D deformation table tbl of size imsize given by coefs c
% which have size csize and are zero everywhere except at cpos
% cpos=x in 1D and cpos=(component,x,y,..) in 2D,3D; component=0,1,..
%
% Usage: [tbl,dspl]=deftable(csize,cpos,imsize,degree)
%
% dspl is tbl minus identity
%
% Jan Kybic, 1999
% $Id: deftableone.m,v 1.1 1999/11/16 15:19:52 jkybic Exp $

ndim=length(imsize) ; % number of dimensions

if ndim>3 | ndim<1, 
  error('Unsupported dimensionality') ;
end ;

if degree<0 | degree>3,
  error('Unsupported degree') ; 
end ;  

shift=max(0,(degree-1)/2) ;

if ndim>1 & cpos(1)>ndim-1,
  error('Incorrect component in cpos') ; 
end ;


if ndim==1, 
  N=csize ;
  tbl=zeros(imsize,1) ; 
  step=(N-1-2*shift)/(imsize-1) ;
  t=shift:step:N-1-shift ;
  idt=(0:imsize-1)' ;
  dspl=splninterpone(cpos,csize,t,ndim,degree)' ;
else 
  tbl=zeros(imsize) ;
  N=csize ; 
  step=(N-1-2*shift)./(imsize-1) ;
  if ndim==2,
    if step(1)>0, yv=shift:step(1):N(1)-1-shift ;
    else        yv=shift*ones(1,imsize(1)) ;
    end ;
    if step(2)>0, xv=shift:step(2):N(2)-1-shift ;
    else        xv=shift*ones(1,imsize(2)) ;
    end ;
    
    [x,y]=meshgrid(xv,yv) ;
    t=zeros(2,imsize(1),imsize(2)) ;
    t(1,:,:)=y ; t(2,:,:)=x ;
    [x,y]=meshgrid(0:imsize(2)-1,0:imsize(1)-1) ;
    idt=zeros(2,imsize(1),imsize(2)) ;
    idt(1,:,:)=y ; idt(2,:,:)=x ;
    dspl=zeros([ ndim imsize ]) ;
    dspl(cpos(1)+1,:,:)=splninterpone(cpos(2:3),csize,t,ndim,degree) ;
  else % ndim==3
    xv=shift:step(1):N(1)-1-shift ;
    yv=shift:step(2):N(2)-1-shift ;
    zv=shift:step(3):N(3)-1-shift ;
    [x,y,z]=meshgrid(yv,xv,zv) ;
    t=zeros(3,length(xv),length(yv),length(zv)) ;
    t(1,:,:,:)=y ; t(2,:,:,:)=x ; t(3,:,:,:)=z ;
    [x,y,z]=meshgrid(0:imsize(2)-1,0:imsize(1)-1,0:imsize(3)-1) ;
    idt=zeros([ ndim imsize ]) ; 
    idt(1,:,:,:)=y ; idt(2,:,:,:)=x ; idt(3,:,:,:)=z ; 
    dspl=zeros([ ndim imsize ]) ;
    dspl(cpos(1)+1,:,:,:)=splninterpone(cpos(2:4),csize,t,ndim,degree) ;
  end ;
end ;

tbl=dspl+idt ;

  
