function [c,T,d]=lssplnapr(x,y,N,deg)
% approximates data x,y by a spline of degree deg described by N
% uniformly placed coefficients for B-splines. The approximation is
% f(x)=sum_{i=1}^N c_i beta_deg(x/T-i+d)
%
% Jan Kybic, February 2000
% $Id: lssplnapr.m,v 1.1 2000/02/01 15:30:24 jkybic Exp $

% calculate T,d, so as to put enough knots outside the interval as not to
% impose any additional constraints
x=x(:) ; y=y(:) ;
xmin=min(x) ; xmax=max(x) ;
T=(xmax-xmin)/(N-deg) ; 
d=N+0.5-deg/2-xmax/T ;

m=length(x) ;
% assemble system matrix
A=zeros(m,N) ;
for k=1:N, % for all unknows
  t=x/T-k+d ;
  A(:,k)=evalbspline(t,deg)' ;
end ;

% solve in the least squares sense
c=A\y ;
