function [c,T,d]=rbfsplnapr(x,y,N,lambda)
% approximates data x,y by a linear combination of |x|^7
% in order to calculate the second derivative
% uniformly placed coefficients for B-splines. The approximation is
% f(x)=sum_{i=1}^N c_i |x-i/T+d|^7+ c_{N+1}+c_{N+2} x+...+c_{N+4} x^3
%
% The approximation minimizes
% J(f)=sum_{k=1}^M (f(x_k)-y_k)^2 + lambda \int (f'''')^2 dx
%
% Jan Kybic, March 2000
% $Id: rbfsplnapr.m,v 1.2 2000/03/27 16:52:11 jkybic Exp $

x=x(:) ; y=y(:) ;
xmin=min(x) ; xmax=max(x) ;

T=(N-1)/(xmax-xmin) ;
d=-xmin+1/T ;

m=length(x) ;
% assemble system matrix
NN=N+4 ;
A=zeros(NN,NN) ; 
r=zeros(NN,1) ; % right hand side 

bv=zeros(m,NN) ; % precalculated values of the generating function
for j=1:N,
  tj=x-j/T+d ; 
  bv(:,j)=abs(tj).^7 ;
end ;
bv(:,N+1)=ones(m,1) ;
bv(:,N+2)=x ;
bv(:,N+3)=x.^2 ;
bv(:,N+4)=x.^3 ;

% add the terms for the data criterion  
for i=1:NN, % for all unknows - lines
  for k=1:NN, % for all unknowns - columns
    A(i,k)=bv(:,i)'*bv(:,k) ;
  end ;  
  r(i)=bv(:,i)'*y ;
end ;

% add the regularization

lf=1680*lambda ;

for i=1:N,
  for k=1:N,
    it=i/T-d ; kt=k/T-d ;
    A(i,k)=A(i,k)+lf*intabs(it,kt,xmin,xmax);
  end ;
end ;

    
% solve the set
%[u,s,v]=svd(A) ;
%s=diag(s) ; q=(s(1)*1e-20+1e-50) ;
%for i=1:length(s),
% if s(i)<q, s(i)=0 ; 
%  else s(i)=1/s(i) ; end ;
%end ;
%c=v*diag(s)*u'*r ;
c=A\r ;
keyboard

function val=intktit(it,kt,xmin,xmax) ;
% Integrates (x-it)^3*(x-kt)^3 from xmin to xmax
%
a=-(it+kt) ; b=it*kt ;
val=intxab(a,b,xmax)-intxab(a,b,xmin) ;

function val=intxab(a,b,x) ;
% Calculates the primitive function to (x^2+a x+b) at point x
val=x*(b^3+x*(1.5*a*b^2+x*(b*(a^2+b)+...
    x*(a/4*(a^2+6*b)+x*(0.6*(a^2+b)+x*(a/2+x/7)))))) ;

function val=intabs(it,kt,xmin,xmax)
% Calculate integral abs(x-it)^3*abs(x-kt)^3 from xmin to xmax
if it>kt, % ensure that kt>=it
  tmp=it ; it=kt ; kt=tmp ;
end ;
if it<=xmin 
  if kt<=xmax,
    val=-intktit(it,kt,xmin,kt)+intktit(it,kt,kt,xmax) ;
  else
    val=-intktit(it,kt,xmin,xmax) ;
  end ;
else
  if kt<=xmax,
    val=intktit(it,kt,xmin,it)-intktit(it,kt,it,kt)+intktit(it,kt,kt,xmax) ;
  else
    val=intktit(it,kt,xmin,it)-intktit(it,kt,it,xmax) ;
  end ;
end ;
