/* Evaluate the SSD criterion, its gradient and Hessian
   Called from mexsdcgr, see there for a description.

   Jan Kybic, 1999
   $Id: sdcgr.c,v 1.22 2000/10/27 17:37:54 jkybic Exp $
*/

#include <math.h>
#include "bsplneval.c"
#include "splninterp.c"

#define TABSIZE 10 /* max support of splines */

void sdcgr(double *fr,double *ftc,double *c,double *E,
		   double *gr,double *H,int degft, int degc, 
	   int frx,int fry,int frz,
           int ftx,int fty,int ftz,
           int cx,int cy,int cz,int nouts) 
{ double *g,e,*coord,*fw,*ccut ; 
  int ix,iy,iz ; int ndim,cnum,imcnt ;
  double *gx,*gy,*gz ; 
  double shift,stepx,stepy,stepz ;
  double parx,pary,parz ;
  double *derx,*dery,*derz ;
  double *dderx,*ddery,*dderz,*dderxy ;
  double *dderxz,*dderyz ;
  int lowx,lowy,lowz,highx,highy,highz ;
  int supp,ind1,ind2,ind3,ind4,ind5 ;
  int jx,jy,jz,kx,ky,kz,jxf,jyf,kxf,kyf ;
  int cnd ;
  double tabx[TABSIZE],taby[TABSIZE],tabz[TABSIZE] ;
  int tabfx[TABSIZE],tabfy[TABSIZE],tabfz[TABSIZE] ;
  double ederx,edderx,bedderx ;
  double betajx,bedderkxx ;
  double edery,eddery,edderxy ; 
  double ederz,edderz,edderxz,edderyz ;
  double betajy,betajxjy,betajxjyky,betajxjykykx ;
  double betajz,betajxjykykz ;
  double h1,h2,h3,h4,h5,h6 ;
  double (*evspln)(double) ;    /* pointer to the evaluation function */
  double beta ;

  ndim=cy>1 ? (cz>1 ? 3 : 2) : 1 ;
  shift=max(0,(degc-1)/2.0) ;
  stepx=(cx-1-2*shift)/(frx-1) ; stepy=(cy-1-2*shift)/(fry-1) ; 
  stepz=(cz-1-2*shift)/(frz-1) ;
  cnum=cx*cy*cz ;
  cnd=cnum*ndim ;
  imcnt=frx*fry*frz ;

  mexPrintf("sdcgr called with fr=(%d,%d,%d) ft=(%d,%d,%d) c=(%d,%d,%d) "
            "ndim=%d shift=%g cnum=%d imcnt=%d nouts=%d\n",frx,fry,frz,
	    ftx,fty,ftz,cx,cy,cz,ndim,shift,cnum,imcnt,nouts) ; 

  if (choosespln(degc,&evspln,&supp,0)) myErrMsg("Unsupported degc") ;

  /* First, let us create the deformation matrix */
  myMalloc(g,ndim,imcnt)  ;
  myMalloc(coord,ndim,imcnt) ;

  switch (ndim) {
  case 1:
    for(ix=0;ix<frx;ix++) coord[ix]=shift+stepx*ix ;
    splinterp(c,coord,g,cx,1,1,frx,1,1,1,degc,0) ; 
    for(ix=0;ix<frx;ix++) g[ix]+=ix ; /* add identity */
    /* Calculate derivatives */
    myMalloc(derx,imcnt,1) ;
    splinterp(ftc,g,derx,ftx,fty,ftz,frx,fry,frz,ndim,degft,1) ;
    myMalloc(dderx,imcnt,1) ;
    splinterp(ftc,g,dderx,ftx,fty,ftz,frx,fry,frz,ndim,degft,2) ;
    break ;
  case 2:
    // mexPrintf("make deformation table\n") ;
    myMalloc(gx,1,imcnt) ; myMalloc(gy,1,imcnt) ;
    myMalloc(ccut,1,cnum) ;
    for(iy=0;iy<fry;iy++) 
      for(ix=0;ix<frx;ix++) { 
	coord[2*(ix+frx*iy)]=shift+stepx*ix ;
	coord[1+2*(ix+frx*iy)]=shift+stepy*iy ;
      }
    /* cut out x-component of c and generate x-component of g */
    for(ix=0;ix<cnum;ix++) ccut[ix]=c[2*ix] ;
    splinterp(ccut,coord,gx,cx,cy,1,frx,fry,1,2,degc,0) ;
    /* cut out y-component of c and generate y-component of g */
    for(ix=0;ix<cnum;ix++) ccut[ix]=c[1+2*ix] ;
    splinterp(ccut,coord,gy,cx,cy,1,frx,fry,1,2,degc,0) ;
    /* assemble g and add identity */
    for (iy=0;iy<fry;iy++)
      for(ix=0;ix<frx;ix++) { 
	g[2*(ix+frx*iy)]  =gx[ix+frx*iy]+ix ; 
        g[1+2*(ix+frx*iy)]=gy[ix+frx*iy]+iy ; 
	// mexPrintf("x=%f y=%f\n",g[2*(ix+frx*iy)],g[1+2*(ix+frx*iy)]) ;
      }
    myFree(gx) ; myFree(gy) ; myFree(ccut) ;
    // mexPrintf("calculate derivatives\n") ;
    /* Calculate derivatives */
    myMalloc(derx,imcnt,1) ;
    splinterp(ftc,g,derx,ftx,fty,ftz,frx,fry,frz,ndim,degft,1) ;
    myMalloc(dery,imcnt,1) ;
    splinterp(ftc,g,dery,ftx,fty,ftz,frx,fry,frz,ndim,degft,4) ;
    myMalloc(dderx,imcnt,1) ;
    splinterp(ftc,g,dderx,ftx,fty,ftz,frx,fry,frz,ndim,degft,2) ; 
    myMalloc(ddery,imcnt,1) ;
    splinterp(ftc,g,ddery,ftx,fty,ftz,frx,fry,frz,ndim,degft,8) ; 
    myMalloc(dderxy,imcnt,1) ;
    splinterp(ftc,g,dderxy,ftx,fty,ftz,frx,fry,frz,ndim,degft,5) ;
    break ;
  case 3:
    // mexPrintf("make deformation table\n") ;
    myMalloc(gx,1,imcnt) ; myMalloc(gy,1,imcnt) ; myMalloc(gz,1,imcnt) ;
    myMalloc(ccut,1,cnum) ;
    for(iz=0;iz<frz;iz++)
      for(iy=0;iy<fry;iy++) 
	for(ix=0;ix<frx;ix++) { 
	  ind1=ix+frx*(iy+fry*iz) ;
	  coord[3*ind1]=shift+stepx*ix ;
	  coord[1+3*ind1]=shift+stepy*iy ;
	  coord[2+3*ind1]=shift+stepz*iz ;
	}
    /* cut out x-component of c and generate x-component of g */
    for(ix=0;ix<cnum;ix++) ccut[ix]=c[3*ix] ;
    splinterp(ccut,coord,gx,cx,cy,cz,frx,fry,frz,3,degc,0) ;
    /* cut out y-component of c and generate y-component of g */
    for(ix=0;ix<cnum;ix++) ccut[ix]=c[1+3*ix] ;
    splinterp(ccut,coord,gy,cx,cy,cz,frx,fry,frz,3,degc,0) ;
    /* cut out z-component of c and generate z-component of g */
    for(ix=0;ix<cnum;ix++) ccut[ix]=c[2+3*ix] ;
    splinterp(ccut,coord,gz,cx,cy,cz,frx,fry,frz,3,degc,0) ;
    /* assemble g and add identity */
    for(iz=0;iz<frz;iz++)
      for (iy=0;iy<fry;iy++)
	for(ix=0;ix<frx;ix++) { 
	  ind1=ix+frx*(iy+fry*iz) ;
	  g[3*ind1]  =gx[ind1]+ix ; 
	  g[1+3*ind1]=gy[ind1]+iy ; 
	  g[2+3*ind1]=gz[ind1]+iz ; 
	}
    myFree(gx) ; myFree(gy) ; myFree(gz) ; myFree(ccut) ;
    // mexPrintf("calculate derivatives\n") ;
    /* Calculate derivatives */
    if (nouts>1) { 
      myMalloc(derx,imcnt,1) ;
      splinterp(ftc,g,derx,ftx,fty,ftz,frx,fry,frz,ndim,degft,1) ;
      myMalloc(dery,imcnt,1) ;
      splinterp(ftc,g,dery,ftx,fty,ftz,frx,fry,frz,ndim,degft,4) ;
      myMalloc(derz,imcnt,1) ;
      splinterp(ftc,g,derz,ftx,fty,ftz,frx,fry,frz,ndim,degft,16) ;
    }
    /* and second derivatives */
    if (nouts>2) {
      myMalloc(dderx,imcnt,1) ;
      splinterp(ftc,g,dderx,ftx,fty,ftz,frx,fry,frz,ndim,degft,2) ; 
      myMalloc(ddery,imcnt,1) ;
      splinterp(ftc,g,ddery,ftx,fty,ftz,frx,fry,frz,ndim,degft,8) ; 
      myMalloc(dderz,imcnt,1) ;
      splinterp(ftc,g,dderz,ftx,fty,ftz,frx,fry,frz,ndim,degft,32) ; 
      myMalloc(dderxy,imcnt,1) ;
      splinterp(ftc,g,dderxy,ftx,fty,ftz,frx,fry,frz,ndim,degft,5) ;
      myMalloc(dderxz,imcnt,1) ;
      splinterp(ftc,g,dderxz,ftx,fty,ftz,frx,fry,frz,ndim,degft,17) ;
      myMalloc(dderyz,imcnt,1) ;
      splinterp(ftc,g,dderyz,ftx,fty,ftz,frx,fry,frz,ndim,degft,20) ;
    }
    break ;

default: myErrMsg(NAME ": Unsupported dimensionality") ;
  }
  myFree(coord) ;

  // mexPrintf("Warp test image\n") ;
  /* Warp the test image */
  myMalloc(fw,imcnt,1) ;
  splinterp(ftc,g,fw,ftx,fty,ftz,frx,fry,frz,ndim,degft,0) ;

  /* zero E,gr and H */
  if (nouts>1) for(ix=0;ix<cnd;ix++) gr[ix]=0.0 ;
  if (nouts>2) for(ix=0;ix<cnd*cnd;ix++) H[ix]=0.0 ;
  *E=0.0 ; 

  /* Calculate the difference and the SSD criterion */
  /* calculate criterion */
  switch (ndim) {
  case 1: /* 1D case gradient and Hessian calculation starts here */
    for(ix=0;ix<frx;ix++) { /* loop through all pixels */
      e=fw[ix]-fr[ix] ;
      (*E)+=e*e ;
      if (nouts<2) continue ;
      /* calculate the range of influential parameters */
      parx=shift+stepx*ix ;
      lowx=(int)floor(parx-shift) ; highx=lowx+degc ;
      ederx=2.0*e*derx[ix] ;
      edderx=2.0*(e*dderx[ix]+derx[ix]*derx[ix]) ;
      for(jx=lowx;jx<=highx;jx++) { /* loop through influential c's */
	betajx=tabx[jx-lowx]=(*evspln)(parx-jx) ;	
	jxf=mfold(jx,cx) ;
	gr[jxf]+=betajx*ederx ;
	/* update also the Hessian matrix */
	if (nouts<3) continue ;  
	bedderx=edderx*betajx ;
	for(kx=lowx;kx<=jx;kx++) { /* loop through influential c's */
	  bedderkxx=bedderx*tabx[kx-lowx] ;
	  kxf=mfold(kx,cx) ;
	  H[kxf+jxf*cnum]+=bedderkxx ; 
	  if (kxf!=jxf) H[jxf+kxf*cnum]+=bedderkxx ;
	} /* for kx */
      } /* for jx */
    } /* for ix */
    break ;

  case 2: /* 2D case gradient and Hessian calculation starts here */
    // mexPrintf("gr and Hessian\n") ;
      for(iy=0;iy<fry;iy++) { /* loop through all pixels */
	pary=shift+stepy*iy ;
	lowy=(int)floor(pary-shift) ; highy=lowy+degc ;
	for(ix=0;ix<frx;ix++) { 
	  ind1=ix+frx*iy ;
	  e=fw[ind1]-fr[ind1] ;
	  (*E)+=e*e ;
	  //mexPrintf("ind=%d fw[ind]=%f fr[ind]=%f e=%f E=%f\n",ind1,
	  //	    fw[ind1],fr[ind1],e,*E) ;
	  if (nouts<2) continue ;
	  /* calculate the range of influential parameters */
	  parx=shift+stepx*ix ; 
	  lowx=(int)floor(parx-shift) ; highx=lowx+degc ;
	  ederx=2.0*e*derx[ind1] ;
	  edery=2.0*e*dery[ind1] ;
	  edderx=2.0*(e*dderx[ind1]+derx[ind1]*derx[ind1]) ;
	  eddery=2.0*(e*ddery[ind1]+dery[ind1]*dery[ind1]) ;
	  edderxy=2.0*(e*dderxy[ind1]+derx[ind1]*dery[ind1]) ;
	  /* precalculate tabx */
	  for(jx=lowx;jx<=highx;jx++) tabx[jx-lowx]=(*evspln)(parx-jx) ;
	  for(jy=lowy;jy<=highy;jy++) taby[jy-lowy]=(*evspln)(pary-jy) ;
	  for(jy=lowy;jy<=highy;jy++) { /* loop through influential c's */
	    betajy=taby[jy-lowy] ;
	    jyf=mfold(jy,cy) ;
	    for(jx=lowx;jx<=highx;jx++) { 
	      betajxjy=tabx[jx-lowx]*betajy ;
	      jxf=mfold(jx,cx) ;
	      ind2=jxf+cx*jyf ;
	      gr[2*ind2]+=betajxjy*ederx ;
	      gr[1+2*ind2]+=betajxjy*edery ;
	      if (nouts<3) continue ;
	      /* update also the Hessian matrix */
	      for(ky=lowy;ky<=highy;ky++) { /* loop through influential c's */
		kyf=mfold(ky,cy) ; 
		betajxjyky=betajxjy*taby[ky-lowy] ; 
		for(kx=lowx;kx<=highx;kx++) {
		  kxf=mfold(kx,cx) ; 
		  ind3=kxf+cx*kyf ;
		  betajxjykykx=betajxjyky*tabx[kx-lowx] ;
		  h1=edderx*betajxjykykx ;
		  h2=edderxy*betajxjykykx ;
		  h3=eddery*betajxjykykx ;
		  H[(2*ind3+0)+(2*ind2+0)*cnd]+=h1 ; 
		  H[(2*ind3+1)+(2*ind2+0)*cnd]+=h2 ; 
		  H[(2*ind3+0)+(2*ind2+1)*cnd]+=h2 ; 
		  H[(2*ind3+1)+(2*ind2+1)*cnd]+=h3 ; 
		} /* for kx */
	      } /* for ky */
	    } /* for jx */
	  } /* for jy */
	} /* for ix */
      } /* for iy */
      break ;

  case 3: /* 3D case gradient and Hessian calculation starts here */
    // mexPrintf("gr and Hessian\n") ;
      for(iz=0;iz<frz;iz++) { /* loop through all pixels */
	parz=shift+stepz*iz ;
	lowz=(int)floor(parz-shift) ; highz=lowz+degc ;
	for(iy=0;iy<fry;iy++) { 
	  //	  mexPrintf("iz=%d parz=%g iy=%d lowz=%d highz=%d\n",iz,parz,iy,lowz,highz) ;
	  pary=shift+stepy*iy ;
	  lowy=(int)floor(pary-shift) ; highy=lowy+degc ;
	  for(ix=0;ix<frx;ix++) { 
	    ind1=ix+frx*(iy+fry*iz) ;
	    e=fw[ind1]-fr[ind1] ;
	    (*E)+=e*e ;
	    if (nouts<2) continue ;
	    /* calculate the range of influential parameters */
	    parx=shift+stepx*ix ; 
	    lowx=(int)floor(parx-shift) ; highx=lowx+degc ;
	    ederx=2.0*e*derx[ind1] ;
	    edery=2.0*e*dery[ind1] ;
	    ederz=2.0*e*derz[ind1] ;
	    if (nouts>2) {
	      edderx=2.0*(e*dderx[ind1]+derx[ind1]*derx[ind1]) ;
	      eddery=2.0*(e*ddery[ind1]+dery[ind1]*dery[ind1]) ;
	      edderz=2.0*(e*dderz[ind1]+derz[ind1]*derz[ind1]) ;
	      edderxy=2.0*(e*dderxy[ind1]+derx[ind1]*dery[ind1]) ;
	      edderxz=2.0*(e*dderxz[ind1]+derx[ind1]*derz[ind1]) ;
	      edderyz=2.0*(e*dderyz[ind1]+dery[ind1]*derz[ind1]) ;
	    }
	  /* precalculate tabx,taby,tabz,tabfx,tabfy,tabfz */
	    for(jx=lowx;jx<=highx;jx++) { 
	      tabx[jx-lowx]=(*evspln)(parx-jx) ; 
	      tabfx[jx-lowx]=3*mfold(jx,cx) ;
	    }
	    for(jy=lowy;jy<=highy;jy++) { 
	      taby[jy-lowy]=(*evspln)(pary-jy) ; 
	      tabfy[jy-lowy]=3*cx*mfold(jy,cy) ;
	    }
	    for(jz=lowz;jz<=highz;jz++) { 
	      tabz[jz-lowz]=(*evspln)(parz-jz) ; 
	      tabfz[jz-lowz]=3*cx*cy*mfold(jz,cz) ;
	    }


	    for(jz=0;jz<=degc;jz++) { /* loop through influential c's */
	      betajz=tabz[jz] ;
	      for(jy=0;jy<=degc;jy++) { 
		betajy=betajz*taby[jy] ;
		for(jx=0;jx<=degc;jx++) { 
		  betajxjy=tabx[jx]*betajy ;
		  ind2=tabfx[jx]+tabfy[jy]+tabfz[jz] ;
		  gr[ind2]+=betajxjy*ederx ;
		  gr[1+ind2]+=betajxjy*edery ;
		  gr[2+ind2]+=betajxjy*ederz ;
		  if (nouts<3) continue ;
		  ind4=ind2*cnd ;
		  /* update also the Hessian matrix */
		  for(kz=0;kz<=degc;kz++) { /* loop through c's again */
		    betajxjykykz=betajxjy*tabz[kz] ; 
		    for(ky=0;ky<=degc;ky++) { 
		      betajxjyky=betajxjykykz*taby[ky] ; 
		      ind5=tabfy[ky]+tabfz[kz] ;
		      for(kx=0;kx<=degc;kx++) {
			ind3=tabfx[kx]+ind5 ;
			betajxjykykx=betajxjyky*tabx[kx] ;
			H[ind3+ind4]+=edderx*betajxjykykx ; 
			H[ind3+1+ind4]+=(h2=edderxy*betajxjykykx) ; 
			H[ind3+2+ind4]+=(h3=edderxz*betajxjykykx)  ; 
			H[ind3+ind4+cnd]+=h2 ; 
			H[ind3+1+ind4+cnd]+=eddery*betajxjykykx ; 
			H[ind3+2+ind4+cnd]+=(h5=edderyz*betajxjykykx) ; 
			H[ind3+ind4+2*cnd]+=h3 ; 
			H[ind3+1+ind4+2*cnd]+=h5 ; 
			H[ind3+2+ind4+2*cnd]+=edderz*betajxjykykx ; 
		      } /* for kx */
		    } /* for ky */
		  } /* for kz */
		} /* for jx */
	      } /* for jy */
	    } /* for jz */
	  } /* for ix */
	} /* for iy */
      } /* for iz */
      break ;
  } /* switch (ndim) */
  // mexPrintf("Freeing\n") ;
  
  switch (ndim) {
  case 3: if (nouts>1) { myFree(derz) ; myFree(derx) ; myFree(dery) ; }
          if (nouts>2) { myFree(dderz) ;  myFree(dderx) ; myFree(ddery) ; 
          myFree(dderxz) ; myFree(dderyz) ;}
	  break ;
  case 2: myFree(dery) ; myFree(ddery) ;  myFree(dderxy) ;
  case 1: myFree(derx) ; myFree(dderx) ; 
      }

  myFree(fw) ;  myFree(g) ;   
  mexPrintf("E=%f\n",*E) ;
}

/* end of sdcgr.c */
