function [E,gr,H]=sdcgr(fr,ftc,c,degft,degc)
% Calculates the SSD criterion and its gradient
% fr - reference image, ftc - test image B-spline coefs
% c - deformation coefs. degft, degc - spline degrees
% H - Hessian, second derivatives of E with respect to c
%
% Usage: [E,gr,H]=sdcgr(fr,ftc,c,degft,degc)
%
% Jan Kybic, 1999
% $Id: sdcgr.m,v 1.14 1999/11/20 19:01:19 jkybic Exp $

imsize=size(ftc) ;          % get image size
ndim=length(imsize) ;       % dimensionality

if ndim==2 & prod(imsize)==max(imsize), 
  ndim=1 ; imsize=max(imsize) ;
end ;

if ndim>3 | ndim<1, 
  error('Unsupported dimensionality') ;
end ;

if degft<0 | degft>3 | degc<0 | degc>3,
  error('Unsupported degree') ; 
end ;  

%cc=cputime ;
g=deftable(c,imsize,degc) ; % calculate deformation matrix g
%disp([ 'Deftable took ' num2str(cputime-cc) ' s' ]) ;
%cc=cputime ;  
ftw=mexsplninterp(ftc,g,ndim,degft) ; % warp test image
%disp([ 'Warping took ' num2str(cputime-cc) ' s' ]) ;
e=ftw-fr ;                  % difference image
E=sum(sum(sum(e.*e))) ;     % the criterion

%cc=cputime ;
% calculate first derivatives
if ndim==1,
   ftd=mexsplninterp(ftc,g,ndim,degft,1) ;
elseif ndim==2,
   ftd=zeros([2 imsize]) ;
   ftd(1,:,:)=mexsplninterp(ftc,g,ndim,degft,1) ;
   ftd(2,:,:)=mexsplninterp(ftc,g,ndim,degft,4) ;
elseif ndim==3,
   ftd=zeros([3 imsize]) ;
   ftd(1,:,:,:)=mexsplninterp(ftc,g,ndim,degft,1) ;
   ftd(2,:,:,:)=mexsplninterp(ftc,g,ndim,degft,4) ;
   ftd(3,:,:,:)=mexsplninterp(ftc,g,ndim,degft,16) ;
end ;   
%disp([ 'Derivatives of ft took ' num2str(cputime-cc) ' s' ]) ;

% calculate second derivatives, just one dimensional
%cc=cputime ;
if ndim==1,
   ftdd=mexsplninterp(ftc,g,ndim,degft,2) ;
elseif ndim==2,
   ftdd=zeros([2 imsize]) ;
   ftdd(1,:,:)=mexsplninterp(ftc,g,ndim,degft,2) ;
   ftdd(2,:,:)=mexsplninterp(ftc,g,ndim,degft,8) ;
elseif ndim==3,
   ftdd=zeros([3 imsize]) ;
   ftdd(1,:,:,:)=mexsplninterp(ftc,g,ndim,degft,2) ;
   ftdd(2,:,:,:)=mexsplninterp(ftc,g,ndim,degft,8) ;
   ftdd(3,:,:,:)=mexsplninterp(ftc,g,ndim,degft,32) ;
end ;   
%disp([ 'Second derivatives of ft took ' num2str(cputime-cc) ' s' ]) ;


csize=size(c) ;
cnum=prod(csize) ; % how many parameters are there

% reshaping, because stupid Matlab cannot work with more than 2D sparse
% matrices
imcnt=prod(imsize) ;
ftd=reshape(ftd,imcnt*ndim,1) ;
ftdd=reshape(ftdd,imcnt*ndim,1) ;
e=reshape(e,imcnt,1) ;

cc1=cputime ;

% Precompute dspl matrices of Bspln values for all the coefs
%cc=cputime ;
dcell=zeros(imcnt*ndim,cnum) ; 
for i=1:cnum, % for all parameters c
  % cc=cputime ;
  if ndim==1,
    [tg,dspl]=deftableone(max(csize),i-1,imsize,degc) ;
  elseif ndim==2,
    [i1,i2,i3]=ind2sub(csize,i) ;
    [tg,dspl]=deftableone(csize(2:3),[i1 i2 i3]-1,imsize,degc) ;
  else
    [i1,i2,i3,i4]=ind2sub(csize,i) ;
    [tg,dspl]=deftableone(csize(2:4),[i1 i2 i3 i4]-1,imsize,degc) ;
  end ;
  dsplsize=size(dspl) ;
  dcell(:,i)=sparse(reshape(dspl,imcnt*ndim,1)) ;
end ;
%disp([ 'Precomputing dspl took ' num2str(cputime-cc) ' s' ]) ;

% Calculate gradient and Hessian
%cc=cputime ;
[gr,H]=privsdcgr(e,dcell,ftd,ftdd,cnum,ndim,imcnt) ;
gr=reshape(gr,csize) ; 
H=reshape(H,[csize csize]) ;
%disp([ 'Hessian summing took ' num2str(cputime-cc) ' s' ]) ;
disp([ 'Gradient+Hessian took ' num2str(cputime-cc1) ' s' ]) ;

