function y=splninterpone(b,insize,coord,dim,degree,dflag)
% Boundary conditions compliant mexsplninterpone
%
% Samples a B-spline of degree `degree', centered at `b' on a grid of
% size `insize', mirror boundary conditions. Evaluates it at all
% points in the coord matrix, each coordinate having dim
% components. Stores results in outp. If `dflag' is not given or zero,
% evaluates the value. Otherwise: dflag=1,4,16 derivative with respect
% to x,y,z dflag=2,8,32 second derivative with respect to x,y,z
%
% Usage: y=splninterpone(b,insize,coord,dim,degree,[dflag])

if nargin==5, dflag=0 ; end ;

if dim==3,
  y=doonez(b,insize,coord, dim, degree,dflag) ;
elseif dim==2,
  y=dooney(b,insize,coord, dim, degree,dflag) ;
elseif dim==1,
  y=doonex(b,insize,coord, dim, degree,dflag) ;
end ;

function f=mfold(k,n) 
m=2*(n-1) ; f=rem(k,m) ;
if k<0, f=f+m ; end ;
if f>=n, f=m-f ; end ;

function y=doonex(b,insize,coord, dim, degree,dflag) ;
supp=degree+1 ; hsupp=supp/2 ;
b(1)=mfold(b(1),insize(1)) ;
y=mexsplninterpone(b,insize,coord, dim, degree,dflag) ;
if b(1)<hsupp & b(1)>0,
  bt=b ; bt(1)=-b(1) ;
  y=y+mexsplninterpone(bt,insize,coord, dim, degree,dflag) ;
end ;
if b(1)>insize(1)-1-hsupp & b(1)<insize(1)-1,
  bt=b ; bt(1)=2*(insize(1)-1)-b(1) ;
  y=y+mexsplninterpone(bt,insize,coord, dim, degree,dflag) ;
end ;

function y=dooney(b,insize,coord, dim, degree,dflag) ;
supp=degree+1 ; hsupp=supp/2 ;
b(2)=mfold(b(2),insize(2)) ;
y=doonex(b,insize,coord, dim, degree,dflag) ;
if b(2)<hsupp & b(2)>0,
  bt=b ; bt(2)=-b(2) ;
  y=y+doonex(bt,insize,coord, dim, degree,dflag) ;
end ;
if b(2)>insize(2)-1-hsupp & b(2)<insize(2)-1,
  bt=b ; bt(2)=2*(insize(2)-1)-b(2) ;
  y=y+doonex(bt,insize,coord, dim, degree,dflag) ;
end ;

function y=doonez(b,insize,coord, dim, degree,dflag) ;
supp=degree+1 ; hsupp=supp/2 ;
b(3)=mfold(b(3),insize(3)) ;
y=dooney(b,insize,coord, dim, degree,dflag) ;
if b(3)<hsupp & b(3)>0,
  bt=b ; bt(3)=-b(3) ;
  y=y+dooney(bt,insize,coord, dim, degree,dflag) ;
end ;
if b(3)>insize(3)-1-hsupp & b(2)<insize(3)-1,
  bt=b ; bt(3)=2*(insize(3)-1)-b(3) ;
  y=y+dooney(bt,insize,coord, dim, degree,dflag) ;
end ;

