function out=spspace(img, n, spacein, spaceout,dir)
% SPSPACE    change spline space
%    SPSPACE(IMG, N, SPACEIN, SPACEOUT [, DIR]) changes the space of the 
%    polynomial spline representation using filters of order N;
%
%    IMG is the a matrix that contains the input image.
%
%    SPACEIN  defines the input  space: 'cardinal', 'dual' or 'bspline'.
%    SPACEOUT defines the output space: 'cardinal', 'dual' or 'bspline'.
%
%    For N==0 all spaces are the same and the output equals the input image.
%    For N==1 the cardinal and the bspline space are the same.
%
%    SPSPACE(IMG, N, SPACEIN, SPACEOUT,DIR) changes the space of the 
%    polynomial spline representation using filters of order N
%    only in the direction 'x' or 'y' 
%
%    WARNING! FOR VERY SMALL DIMENSION ROUTINE IS NOT CORRECT DUE TO 
%             BORDER EFFECTS!
%    Reference: IEEE Vol15, No.4, p 371, fig 3.
%    Author: Stefan Horbelt, 23.4.1998
%    Last modification: 18.8.99
%
%    Routines are checked: n<=3 card->bspl->card (lena256) < 1E6 max error 
%    Routines give same values as C implentation: max difference <1E5
%
%    BEWARE for N=4 ca. 0.1% max error = 0.01 dB!
%           for N=5 ca. 0.3% max error = 0.02 dB!
%
%    BEWARE BORDERCONDITIONS!
%
error(nargchk(4,5,nargin))

%% In the trival case n==0 all spaces are the same:
%% cardinal==dual==bspline==orthogonal
if(n==0)
  out=img;
  return;
end

spacein =spacein(1:4);
spaceout=spaceout(1:4);

%% For n==1 the cardinal and the bspline space are the same:
if(n==1)
  if( (spacein=='card' & spaceout=='bspl') | ...
      (spacein=='bspl' & spaceout=='card')  )
    out=img;
    return;
  end
end    
	
    
xsize=size(img,1);
ysize=size(img,2);
% TODO: verify thsi direction: used by resie2D1 rad_filtering.
if(nargin==5)
  if(strcmp(dir,'y'))
    ysize=-ysize;
  elseif(strcmp(dir,'x'))
    xsize=-xsize;
  end
end

% Cardinal -> Bspline: (b^n)^-1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%ok
if(spacein=='card' & max(spaceout~='card'))

  if(ysize>1)
    for x=1:abs(xsize)
      img(x,:)=fspline(img(x,:),n);
    end; 
  end;
  
  if(xsize>1)
    for y=1:abs(ysize)
      img(:,y)=fspline(img(:,y),n);
    end;
  end;
  spacein='bspl';
end;

% Bspline -> Dual : (b^2n+1)^-1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(spacein=='bspl' & spaceout=='dual')

  fb2=fbspline(n*2+1);
  
  if(ysize>1)
    for x=1:abs(xsize)
      img(x,:)=filfirs(img(x,:),fb2);
    end;
  end;
  
  if(xsize>1)
    for y=1:abs(ysize)
      img(:,y)=filfirs(img(:,y),fb2);
    end;
  end;
  spacein='dual';
end;

% Dual->B-spline: (b^(2n+1))^-1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (spacein=='dual' & max(spaceout~='dual'))

%'Dual->bspl'
  
  if(ysize>1)
    for x=1:abs(xsize)
      img(x,:)=fspline( img(x,:),2*n+1);
    end;
  end;
  
  if(xsize>1)
    for y=1:abs(ysize)
      img(:,y)=fspline( img(:,y),2*n+1);
    end;
  end;
  spacein='bspl';
end;

% B-spline->cardinal
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (spacein=='bspl' & spaceout=='card')
%'bspl->dual'

  fb=fbspline(n);
  
  if(ysize>1)
    for x=1:abs(xsize)
      img(x,:)=filfirs(img(x,:),fb);
    end;
  end;
  
  if(xsize>1)
    for y=1:abs(ysize)
      img(:,y)=filfirs(img(:,y),fb);
    end;
  end;
end;

out=img;



