function y=nkernel(N,H,x)
% Y=NKERNEL(N,H,X)  calculates a B-spline N-th kernel
%    The kernel is a convolution of N B-splines of degree [N]
%    where the B-splines has a support of H.*[-(N+1)/2,(N+1)/2].
%
%    The vector H and N contain the H and N values for each dimension. 
%    They must be of same length
%
%    Method: Bionomial sums of one-sided power functions
%
%    Example:
%    [x,y]=draw_nkernel([0 2 1 0],[1 .1 .2 .5],x);      
%
%    Remarks: If all supports H are eual to h then the result is 
%             equivalent to DRAW_BSPLN(SUM(N)+LENGTH(N)-1,h);
%             If any two support h_i=h_j, then we can drop B-spline j
%             and use B-spline i the degree instead: n_i=n_i+n_j+1
%
%    [x,y0]=draw_nkernel([0 2 1 0],[1 .1 .1 .5],x);      
%    [x,y1]=draw_nkernel([0 4 0],  [1 .1 .5],x);      
%    norm(y0-y1)
%
%    See also: KERNEL, XBSPLN, BIKERNEL, TRIKERNEL
%    See also: DEMO_KERNELS, DRAW_BSPLN, DRAW_BIKERNEL, DRAW_TRIKERNEL
%              DRAW_NKERNEL

% >> mcc -V1.2 -ri nkernel.m
error(nargchk(3,3,nargin))

if(prod(H)==0)
  N(H==0)=[];
  H(H==0)=[];
end  

%The faculty of n:
nf=prod([1:(sum(N)+length(N)-1)]);

%The support of x-axis:
support=sum((N+1).*H);
x=x+support/2;

y=x*.0;

%% In order to normalize the surface on the curve to we have
%% to devide by the two smallest support size of h1, h2 and h3:
%mins=sort([H]);
%mins=prod(mins(1:end-1));
mins=prod(H);

%% FOR LOOP K=[0 0 ... 0] TO N
K=N.*0;
n=length(N);
j=1; stop=0;
while(stop==0)
      
    %%% This is the main part of the formula
    y0=1/prod(H.^N)./nf.*...
	((x-sum(K.*H))>0).*...
	(x-sum(K.*H)).^(sum(N)+length(N)-1)./mins;
    for i=1:n	
      y0=y0.*(-1)^K(i).*nchoosek(N(i)+1,K(i));
    end;
    
    y=y+y0;
  
  K(1)=K(1)+1; 
  j=1;
  
  while(K(j)>N(j)+1)
    K(j)=0;
    if(j<n)
      j=j+1;
      K(j)=K(j)+1;
    else
      stop=1;
      break;
    end
  end;

end

% ENDFOR LOOP over K
y=y.*(x<=support).*(x>=0);

