function [E,gr,H]=regcrit(c,par)
% Evaluates a registration criterion and its gradient 
% par contains fr,ftc,alpha
% fr - reference image, ftc - test image coefs (to be deformed)
% c - warping function parameters, alpha - regularization
%
% Usage: [E,gr]=regcrit(c,par)
%
% Jan Kybic, 1999
% $Id: regcrit.m,v 1.20 2000/11/16 14:30:25 jkybic Exp $

fr=par.fr ; ftc=par.ftc ; alpha=par.alpha ; csize=par.csize ;
% warping func and image interp. spline degrees
degree=par.degree ; degim=par.degim ; 

c=reshape(c,csize) ;
imsize=size(fr) ;
if prod(imsize)==max(imsize),
  imsize=max(imsize) ;
end ;

ndim=length(imsize) ;

cc=cputime ;
if 1, % mex
  if nargout>2,
    if 1, % real Hessian
      [E,gr,H]=mexsdcgr(fr,ftc,c,degim,degree) ;
      H=H+eye(prod(csize))*2*alpha ;
      gr=gr+2*alpha*reshape(c,prod(csize),1) ;
    else % simplified Hessian
      [E,gr]=mexsdcgr(fr,ftc,c,degim,degree) ;
      H=gr'*gr ;
      H=H+eye(prod(csize))*2*alpha ;
      gr=gr+2*alpha*reshape(c,prod(csize),1) ;
    end ;
  elseif nargout>1,  
    [E,gr]=mexsdcgr(fr,ftc,c,degim,degree) ;
    gr=gr+2*alpha*reshape(c,prod(csize),1) ;
  else   
    E=mexsdcgr(fr,ftc,c,degim,degree) ;
  end ;
else % non mex - just different output rearrangement
  [E,gr,H]=sdcgr(fr,ftc,c,degim,degree) ;
  gr=gr+2*alpha*c ;
  gr=reshape(gr,prod(csize),1) ;
  H=reshape(H,prod(csize),prod(csize)) ;
  H=H+eye(prod(csize))*2*alpha ;
end ;  

E=E+alpha*sum(sum(sum(sum(c.^2)))) ;
disp([ 'Evaluation took ' num2str(cputime-cc) ' s  ' ...
      num2str(nargout) ' params E=' num2str(E) ]) ;
if 0,
  f=fopen('regcrit.log','a') ;
  fprintf(f,'%g %g\n',cputime,E) ;
  fclose(f) ;
end ;



